#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : ann
# @Time         : 2021/2/1 4:12 下午
# @Author       : yuanjie
# @Email        : meutils@qq.com
# @Software     : PyCharm
# @Description  : mi-ann --conf zk_path
# mongo id mapping

from meutils.pipe import *
from meutils.db.mongo import Mongo
from annzoo.ann import ANN

parser = argparse.ArgumentParser(description='ANN Serive')
parser.add_argument('--conf', default='/push/easyann/user_ann', help='zk/yaml')
args = parser.parse_args()

logger.info(f'cli args: {args.__dict__}')


class Config(BaseConfig):
    data = '/Users/yuanjie/Desktop/vec.parquet'
    reader = ''

    ips: List[str]
    collection_name = 'demo'
    ann_fields: List = Field([], alias='fields')
    auto_id: bool = True
    segment_row_limit: int = 4096
    batch_size = 10000

    # idmapping
    insert_vector: bool = False


conf = Config.parse_yaml(args.conf) if Path(args.conf).is_file() else Config.parse_zk(args.conf)


def insert_data(ip):
    ann = ANN(ip)
    ann.create_collection(
        conf.collection_name,
        fields=conf.ann_fields,
        auto_id=conf.auto_id,
        segment_row_limit=conf.segment_row_limit
    )

    dfs = eval(conf.reader)(conf.data)  # df生成器

    for df in dfs:
        ann_collection = ann.__getattr__(conf.collection_name)
        ids = ann_collection.batch_insert(df, conf.batch_size)

        if 'sid' in df:  # 无sid无需映射

            df['id'] = ids
            df['date'] = datetime.datetime.utcnow()  # ttl

            cols = ['id', 'sid', 'date'] + ['vec'] if conf.insert_vector else []  # 测试性能

            mongo_collection = Mongo(url=None).db.__getattr__(conf.collection_name)
            mongo_collection.insert_many(df[cols].to_dict('r'), False)


def main():
    conf.ips | xThreadPoolExecutor(insert_data, max_workers=3) | xlist


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : klingai
# @Time         : 2024/7/9 13:23
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import jsonpath

from meutils.pipe import *
from meutils.db.redis_db import redis_aclient
from meutils.oss.minio_oss import Minio
from meutils.io.image import image2nowatermark_image
from meutils.schemas.baidu_types import PICINFO2_RIGHT_BOTTOM

from meutils.decorators.retry import retrying
from meutils.schemas import kling_types
from meutils.schemas.kuaishou_types import BASE_URL, UPLOAD_BASE_URL, KlingaiImageRequest, FEISHU_URL, FEISHU_URL_VIP

from meutils.notice.feishu import send_message as _send_message
from meutils.config_utils.lark_utils import get_next_token_for_polling
from meutils.schemas.task_types import Task, FileTask

from meutils.apis.proxy.kdlapi import get_one_proxy

send_message = partial(
    _send_message,
    title=__name__,
    url="https://open.feishu.cn/open-apis/bot/v2/hook/dc1eda96-348e-4cb5-9c7c-2d87d584ca18"
)


@retrying(max_retries=3, title=__name__)  # 触发重试
@alru_cache()
async def upload(file: bytes, filename: Optional[str] = None, cookie: Optional[str] = None,
                 vip: bool = False):  # 应该不绑定cookie

    vip = True  # 上传强制走 vip 保证稳定性

    cookie = cookie or await get_next_token_for_polling(FEISHU_URL_VIP if vip else FEISHU_URL, check_token=check_token)

    filename = filename or f"{shortuuid.random()}.png"  # 文件内容和实际类型不符

    headers = {
        'Cookie': cookie,
        'Content-Type': 'application/json;charset=UTF-8'
    }

    async with httpx.AsyncClient(base_url=UPLOAD_BASE_URL, headers=headers, timeout=100) as client:
        # 文件名生成token
        response = await client.get(f"{BASE_URL}/api/upload/issue/token", params={"filename": filename})
        logger.debug(response.json())

        token = jsonpath.jsonpath(response.json(), "$.data.token")[0]  ####### bool[0]

        # 判断是否存在
        response = await client.get("/api/upload/resume", params={"upload_token": token})
        # {"result":1,"existed":true,"fragment_index":-1,"fragment_list":[],"endpoint":[{"protocol":"KTP","host":"103.107.217.16","port":6666},{"protocol":"KTP","host":"103.102.202.156","port":6666},{"protocol":"TCP","host":"103.107.217.16","port":6666}],"fragment_index_bytes":0,"token_id":"d36ce45c09ce9b84","prefer_http":false}
        logger.debug(response.json())

        # 上传
        response = await client.post(
            "/api/upload/fragment",
            params={"fragment_id": 0, "upload_token": token},
            content=file
        )
        logger.debug(response.json())

        # 校验
        response = await client.post(
            "api/upload/complete",
            params={"fragment_count": 1, "upload_token": token},
        )
        logger.debug(response.json())

        # 最终
        response = await client.get(f"{BASE_URL}/api/upload/verify/token", params={"token": token})
        if response.is_success:
            data = response.json()
            # logger.debug(data)
            # send_message(data)
            # {'status': 200, 'message': '成功', 'data': {'status': 8, 'url': '', 'message': '上传图片包含敏感信息'}, 'timestamp': [2024, 8, 30, 17, 32, 37, 964976000]}
            # {'status': 200, 'message': '成功', 'data': None, 'timestamp': [2024, 8, 30, 17, 32, 37, 964976000]}
            # {'result': 1, 'status': 400, 'message': '文件内容和实际类型不符', 'data': None, 'timestamp': [2024, 9, 26, 18, 26, 29, 838371000]}
            if data['data'] is None:  # todo
                Exception(str(data))

            url = data['data']['url']
            return FileTask(url=url, data=data, system_fingerprint=token)

        response.raise_for_status()


# {
#     "result": 1,
#     "status": 200,
#     "message": "成功",
#     "data": {
#         "status": 3,
#         "url": "https://s22-def.ap4r.com/bs2/upload-ylab-stunt-sgp/ai_portal/1727178999/pOtXQbT4cu/x.png",
#         "message": ""
#     },
#     "timestamp": [
#         2024,
#         9,
#         24,
#         19,
#         56,
#         43,
#         861415000
#     ]
# }


@retrying(max_retries=5, predicate=lambda r: not r, title=__name__)
async def create_task(request: KlingaiImageRequest, token: Optional[str] = None, vip: bool = False):
    cookie = token or await get_next_token_for_polling(FEISHU_URL_VIP if vip else FEISHU_URL, check_token=check_token,
                                                       from_redis=True)

    headers = {
        'Cookie': cookie,
        'Content-Type': 'application/json;charset=UTF-8'
    }

    request_kwargs = {}
    for i in range(3):
        async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=60, **request_kwargs) as client:
            response = await client.post("/api/task/submit", json=request.payload)

            logger.debug(response.status_code)
            logger.debug(response.text)

            if response.is_success:
                data = response.json()
                send_message(data)

                # 触发重试 404 429 520
                if any(i in str(data) for i in {"页面未找到", "请求超限", "配额耗尽", "积分消费失败"}):
                    send_message(f"{data}\n\n{cookie}")
                    # 走代理
                    request_kwargs = {
                        "proxy": await get_one_proxy(from_redis=True),
                        # "proxies": proxies,
                    }
                    return

                try:
                    task_ids = jsonpath.jsonpath(data, "$..task.id")  # $..task..[id,arguments]
                    if task_ids:
                        task_id = task_ids[0]
                        status_code = jsonpath.jsonpath(data, "$..task.status")[0]

                        return Task(id=task_id, status_code=status_code, data=data, system_fingerprint=cookie)
                    else:
                        return Task(status=0, data=data)

                except Exception as e:
                    logger.error(e)
                    send_message(f"未知错误：{e}")


@retrying(max_retries=3, predicate=lambda r: not r, title=__name__)
async def create_task_plus(request: KlingaiImageRequest, token: Optional[str] = None, vip: bool = False):
    token = token or await get_next_token_for_polling(FEISHU_URL_VIP if vip else FEISHU_URL, check_token=check_token,
                                                      from_redis=True)

    if not vip: request.mode = "mini"  # 针对普号

    headers = {
        'Cookie': token,
        'Content-Type': 'application/json;charset=UTF-8'
    }

    request_kwargs = {}
    for i in range(3):
        async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=60, **request_kwargs) as client:
            response = await client.post("/api/task/submit", json=request.payload)
            response.raise_for_status()

            logger.debug(response.status_code)
            logger.debug(response.text)
            # {"result": 1, "status": 416, "message": "任务提交失败", "data": null,
            #  "timestamp": [2024, 9, 24, 20, 8, 56, 132298000]}

            # {"result": 1, "status": 401, "message": "未授权", "data": null,
            #  "timestamp": [2024, 9, 26, 10, 52, 24, 90756000]}

            # {"result": 1, "status": 200, "message": "成功",
            #  "data": {"task": null, "works": [], "status": 7, "message": "输入的提示词包含敏感词",
            #           "limitation": {"type": "mmu_txt2img_aiweb", "remaining": 10000, "limit": 10000}, "userPoints": null},
            #  "timestamp": [2024, 9, 26, 8, 58, 42, 191747000]}

            if response.is_success:
                data = response.json()
                kling_types.send_message(data)

                # 触发重试 404 429 520
                if any(i in str(data) for i in {"页面未找到", "请求超限", "配额耗尽", "积分消费失败"}):
                    kling_types.send_message(f"{data}\n\n{token}")

                    # 走代理
                    request_kwargs = {
                        "proxy": await get_one_proxy(from_redis=True),
                        # "proxies": proxies,
                    }

                    return

                task_id = message = task_status_msg = ''
                code = data.get("status")
                if data.get("data"):
                    data = data.get("data")
                    if task := data.get("task"):
                        code = 0
                        task_id = f"""kling-image-{request.mode}-{task.get("id")}"""
                        task_status = "processing"
                        task_status_msg = data.get("message")
                    else:
                        task_status = "failed"
                        task_status_msg = data.get("message")

                else:
                    message = data.get("message")
                    task_status = "failed"

                return kling_types.TaskResponse(
                    code=code,
                    message=message,
                    data=kling_types.Task(task_id=task_id, task_status=task_status, task_status_msg=task_status_msg),
                    system_fingerprint=token,
                )


@retrying(max_retries=3, exp_base=1.1, predicate=lambda r: r == "RETRYING")  # 触发重试
async def get_task(task_id, cookie: str):
    headers = {
        'Cookie': cookie,
        'Content-Type': 'application/json;charset=UTF-8'
    }

    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=60) as client:
        response = await client.get("/api/task/status", params={"taskId": task_id, "withWatermark": False})
        if response.is_success:
            data = response.json()

            logger.debug(data)

            if not task_id or "failed," in str(data):
                # return "TASK_FAILED"  # 跳出条件
                return data

            urls = jsonpath.jsonpath(data, '$..resource.resource')
            if urls and all(urls):
                images = [{"url": url} for url in urls]
                return images
            else:
                return "RETRYING"  # 重试


@retrying(title="kling-images&videos")  # 触发重试
async def get_task_plus(task_id, token: str, oss: Optional[str] = None):  # image video
    task_id = str(task_id)

    vip = "mini" not in task_id
    task_type = "images" if 'image' in task_id else "videos"

    task_id = task_id.rsplit('-', 1)[-1]

    headers = {
        'Cookie': token,
        'Content-Type': 'application/json;charset=UTF-8'
    }

    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=60) as client:
        response = await client.get("/api/task/status", params={"taskId": task_id, "withWatermark": False})
        response.raise_for_status()

        if response.is_success:
            data = response.json()
            logger.debug(bjson(data))

            works = data['data']['works']
            work_ids = [work['workId'] for work in works]
            urls = [work['resource']['resource'] for work in works]

            if data['data']['status'] == 99:  # 任务完成

                nowatermark_urls = [None] * len(urls)
                if vip:  # 会员获取无水印链接
                    func = partial(download, token=token)
                    nowatermark_urls = await asyncio.gather(*map(func, work_ids))

                if task_type == "images":  # 非会员图片去水印
                    if not all(nowatermark_urls):  # 获取失败就调用去水印
                        func = partial(image2nowatermark_image, oss=oss or "kling", picinfo=PICINFO2_RIGHT_BOTTOM)
                        nowatermark_urls = await asyncio.gather(*map(func, urls))

                if all(nowatermark_urls):
                    for work, url in zip(data['data']['works'], nowatermark_urls):
                        work['resource']['resource'] = url

            return data


@retrying(max_retries=3, predicate=lambda r: not r, title="kling-download")
async def download(work_id, token):
    headers = {
        'Cookie': token,
    }
    params = {
        "workIds": work_id,
        "fwm": False
    }
    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=100) as client:
        response = await client.get("/api/works/batch_download_v2", params=params)
        response.raise_for_status()

        if response.is_success:
            data = response.json()
            logger.debug(bjson(data))

            if any(i in str(data) for i in {"页面未找到", "请求超限", "配额耗尽", "积分消费失败"}):
                kling_types.send_message(f"{data}\n\n{token}")
                return

            if data['data']:
                return data['data']['cdnUrl']


async def image_to_transfer(work, headers):
    try:
        workId = work['workId']
        video_url = work['resource']['resource']

        if video_url and (oss_url := await redis_aclient.get(video_url)):

            logger.info(video_url)
            logger.info(oss_url)

            oss_url = oss_url.decode()
            return oss_url

        elif video_url:
            with timer("转存"):
                no_watermark_url = f"https://klingai.com/api/works/batch_download_v2?workIds={workId}&fwm=false"

                file_object = await Minio().put_object_for_openai(
                    file=no_watermark_url,
                    filename=f"{shortuuid.random()}-{workId}",
                    headers=headers
                )

                oss_url = file_object.filename

                await redis_aclient.set(video_url, oss_url, ex=24 * 3600)
                await redis_aclient.set(oss_url, video_url, ex=24 * 3600)

                return oss_url

    except Exception as e:
        logger.error(e)
        kling_types.send_message(traceback.format_exc(), title=__name__)


async def video_to_transfer(work, headers):
    try:
        workId = work['workId']
        video_url = work['resource']['resource']

        if video_url and (oss_url := await redis_aclient.get(video_url)):

            logger.info(video_url)
            logger.info(oss_url)

            oss_url = oss_url.decode()
            return oss_url

        elif video_url:
            with timer("转存"):
                no_watermark_url = f"https://klingai.com/api/works/batch_download?workIds={workId}&fwm=false"

                file_object = await Minio().put_object_for_openai(
                    file=no_watermark_url,
                    filename=f"{shortuuid.random()}-{workId}",
                    headers=headers
                )

                oss_url = file_object.filename

                await redis_aclient.set(video_url, oss_url, ex=24 * 3600)
                await redis_aclient.set(oss_url, video_url, ex=24 * 3600)

                return oss_url

    except Exception as e:
        logger.error(e)
        kling_types.send_message(traceback.format_exc(), title=__name__)


# {
#     "code": 0,
#     "message": "string",
#     "request_id": "string",
#     "data": {
#         "task_id": "string",
#         "task_status": "string",
#         "task_status_msg": "string",
#         "created_at": 1722769557708,
#         "updated_at": 1722769557708,
#         "task_result": {
#             "videos": [
#                 {
#                     "id": "string",
#                     "url": "string",
#                     "duration": "string"
#                 }
#             ]
#         }
#     }
# }

@retrying(max_retries=3)
async def create_image(request: KlingaiImageRequest):
    vip = True
    task = await create_task(request, vip=vip)

    logger.debug(task)

    data = await get_task(task.id, task.system_fingerprint)

    return data


@alru_cache(ttl=30)
@retrying()
async def get_reward(cookie: str):
    headers = {
        'Cookie': cookie,
        'Content-Type': 'application/json;charset=UTF-8'
    }
    params = {"activity": "login_bonus_daily"}
    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers) as client:
        response = await client.get("/api/pay/reward", params=params)
        if response.is_success:
            data = response.json()
            return data


@alru_cache(ttl=30)
@retrying()
async def get_point(cookie: str):
    headers = {
        'Cookie': cookie,
        'Content-Type': 'application/json;charset=UTF-8'
    }
    await get_reward(cookie)

    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=15) as client:
        response = await client.get("/api/account/point")
        if response.is_success:
            data = response.json()
            return data
        # response.raise_for_status()


async def check_token(token, threshold=10):
    try:
        data = await get_point(token)
        # logger.debug(data['data']['total'])

        return data['data']['total'] >= threshold * 100  # 视频
    except Exception as e:
        logger.error(f"{e}\n无效{token}")
        return False


if __name__ == '__main__':
    token = "weblogger_did=web_4557277593AC34F7; _gcl_au=1.1.2117561676.1724226781; did=web_219408fbb50842a98d7b2d6bdb6549fa98f9; _ga=GA1.1.1030366908.1726724249; _ga_MWG30LDQKZ=GS1.1.1726724249.1.1.1726724301.0.0.0; anonymous-message-release-notice-1=true; dev-center-view-welcome-dialog-key=true; anonymous-message-release-notice-2=true; welcome-future-partner-key=true; userId=3412057; ksi18n.ai.portal_st=ChNrc2kxOG4uYWkucG9ydGFsLnN0EqABBEYYivZgEfNHmCqUd0Ci5C0SDosXztozi006E5w6PqfaFej3uWbtWO8aocYHT5YgiATt4tcAXBXSi2YlhN69uQsHOBnu3rTWmVBfn7mYcIG9UDJEJMk5wncXA35QlX643D2sNtcUHY1aVaWOb5xasMqZLgVj2jnVK1AATd9hy50ECmdhTJg9vo6Zd6DGt5HwfJEF96MwoZf_252x_IUBrBoSs3j9uCe9B29gvrCKIsq-ZxJ7IiCcCFyiXd3GWPdxK1jYqsP1SVc-QQa3H7KNzbISuqtdaigFMAE; ksi18n.ai.portal_ph=f794330811e9af17f0894907aeb67f71d51b"
    arun(check_token(token))

    # 存活率
    # arun(get_next_token_for_polling(feishu_url=FEISHU_URL, check_token=check_token))

    arun(get_next_token_for_polling(feishu_url=FEISHU_URL_VIP, check_token=check_token))

    # https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=v8vcZY
    # url = "https://s22-def.ap4r.com/bs2/upload-ylab-stunt-sgp/ai_portal/1724806149/rXWnzc3Dhv/x.png"
    #
    request = KlingaiImageRequest(
        prompt="笑起来", imageCount=1,
        # url=url
    )  # 27638649
    # token = "monetization-ads-shown-count-xx=T;_gcl_au=1.1.1994862793.1723432037;did=web_b11919c67a1966b83eaef4a19fb2de266cba;ksi18n.ai.portal_ph=4ab564ff72d22ac3c25c0dacd35cbd5d68ca;ksi18n.ai.portal_st=ChNrc2kxOG4uYWkucG9ydGFsLnN0EqABlvpmXr2OeS7mUSuMO-KL1sVJfy5GcuzNso0ZdOKIlmXj42dZqAjU_14VrJV-e2Yrp4OODrbKy-ZioGBZg1pnU0PKbgK_okknTNnVhujftS8xQoIBvwuIQYoErKr6NNrcIgPnwLvZMYZc4oQLCrs0xNlkm_S4ZET44R3T7PJntudwzNuoIT_5QOn0HMja6Q80eon0o4Zw2-ivZevLwNsWghoSp94kUOm9gCAed8BLne8huz5PIiAM29axw-J1X8AUXudFsw-ZaBZY83v2XfR1d6CAz3roaygFMAE;monetization-ads-shown-count-xxx=T;userId=3426808;weblogger_did=web_8897873009A74F8"
    # token = "_gcl_au=1.1.2117561676.1724226781;did=web_219408fbb50842a98d7b2d6bdb6549fa98f9;ksi18n.ai.portal_ph=b597d7d5856b3c29e0b279defe349b0fc88a;ksi18n.ai.portal_st=ChNrc2kxOG4uYWkucG9ydGFsLnN0EqABdJQ_kVVXVCqIczbfFLJT5Swq6ibgU2mkEOe0njB-ElJnZWI1_Ukw3xikycgE9WLxak5tQpCv8YdEBEiTbcu-YxCs8ntqJkrTpAxcP7a-j216wxvUwzlNcmmK12OlXLaXBRRLgsul9XLnff-d91eCjDnY27y0nb7PizdWdgdbJ39G7xuiTnvZar6TH4nAScGrXSFM5fhVCNJJT1f_yT79bRoSn4p1S_AmSvxx8MBv-fWXfBXKIiBNx-fSmLqe0EJl_zEozwLjbzirH0CJj_lqibAoDmUXHygFMAE;userId=3626555;weblogger_did=web_4557277593AC34F7"

    # arun(get_point(token))
    # cookie = "weblogger_did=web_47164250171DB527; did=web_e022fde52721456f43cb66d90a7d6f14e462; userId=742626779; kuaishou.ai.portal_st=ChVrdWFpc2hvdS5haS5wb3J0YWwuc3QSoAGAEPOivL4BJ2Y8y48CvR-t25o44Sj_5G9LnZI8BJbV_Inkqd4qxPMJy4OqZCf0VHZnr8EcgMHOzuj_fw5-x0OF3UtrXrU2ZBe6G_bnD1umPIAL6DVtv6ERJ9uLpa7asCBgIUvMXk6K345vc5okzhoTPw69b1GsXY777qwuOwGoUrP9eyJc6Z4TeQPYDEW2wdazss7Dn2osIhObsW9izb1yGhJaTSf_z6v_i70Q1ZuLG30vAZsiIGMXZhr3i8pOgOICzAXA0T6fJZZk3hFRsxn3MDQzIeiKKAUwAQ; kuaishou.ai.portal_ph=fe74c1e2fb91142f838c4b3d435d6153ccf3"
    # cookie = "did=web_bd7da66e83ea345fac39694d32d4672b9e07;ksi18n.ai.portal_st=ChNrc2kxOG4uYWkucG9ydGFsLnN0EqAB1Xjdnlyrc7pKORRU-g10oEUbejZbRSGuv4CKK6_colUDfWdfBysqENjMM11prWCVJqKCLUKgy9U3XxPD7KVtgd_nEom9gS1TnzFWDYjgnnULYyszQ9C9DPylj9glH_xThIuy9rN6gpLxPnRtTwj2fh7f1Uy_cSzkOQx_Th3ePnpauTOw-KhCE25G6eXteybcjjotKgd2JWKcKd_3QqRIDhoSBRHAf_JILfM_YhcnxIVtU2YmIiBzb2VMb2UCDTac51ufmB9GzIPUXMNvgrqbKQr8GigFpigFMAE;userId=3431862"
    # task = arun(create_task(request))
    #
    # arun(get_task(task.id, task.system_fingerprint))

    # pprint(arun(get_task(46578557)))
    #
    # pprint())

    # file = open("/Users/betterme/PycharmProjects/AI/MeUtils/examples/apis/lzp.jpg", "rb").read()
    # file = open("test.webp", "rb").read()

    # arun(upload(file))

    # print(request)

    # request = KlingImageRequest(
    #     prompt="一条狗", n=4,
    #     # url=url
    # )  # 27638649
    # print(request)
    # arun(generate(request))

    # arun(download(1, 1))



from typing import List, Optional, Union, Dict, Literal
from pydantic import BaseModel

from itertools import groupby


class Url(BaseModel):
    url: str
    detai: str = "auto"


class TextContent(BaseModel):
    type: Literal["text"]
    text: str


class ImageContent(BaseModel):
    type: Literal["image_url"]
    image_url: Url


class VideoContent(BaseModel):
    type: Literal["video_url"]
    video_url: Url


class FileContent(BaseModel):
    type: Literal["file_url"]
    file_url: Url


class Message(BaseModel):
    role: Literal["system", "user", "assistant"]
    content: Union[str, List[Union[TextContent, ImageContent, VideoContent, FileContent]]]


class Messages(BaseModel):
    messages: List[Message]

    def get_last_message(self) -> Optional[Message]:
        """获取最后一条消息"""
        return self.messages[-1] if self.messages else None

    def get_messages_by_role(self, role: str) -> List[Message]:
        """获取指定角色的所有消息"""
        return [msg for msg in self.messages if msg.role == role]

    def get_all_file_urls(self) -> List[str]:
        """获取所有文件URL"""
        file_urls = []
        for msg in self.messages:
            if isinstance(msg.content, list):
                for item in msg.content:
                    if isinstance(item, FileContent):
                        file_urls.append(item.file_url.url)
            elif isinstance(msg.content, FileContent):
                file_urls.append(msg.content.file_url.url)
        return file_urls

    def get_conversation_pairs(self) -> List[tuple]:
        """获取对话对（用户问题和助手回答的配对）"""
        pairs = []
        user_msg = None
        for msg in self.messages:
            if msg.role == 'user':
                user_msg = msg
            elif msg.role == 'assistant' and user_msg:
                pairs.append((user_msg, msg))
                user_msg = None
        return pairs

    def get_text_content(self) -> List[str]:
        """获取所有文本内容"""
        texts = []
        for msg in self.messages:
            if isinstance(msg.content, str):
                texts.append(msg.content)
            elif isinstance(msg.content, list):
                for item in msg.content:
                    if isinstance(item, TextContent):
                        texts.append(item.text)
        return texts

    def get_message_at_index(self, index: int) -> Optional[Message]:
        """获取指定索引的消息"""
        try:
            return self.messages[index]
        except IndexError:
            return None

    def get_message_count(self) -> int:
        """获取消息总数"""
        return len(self.messages)

    def get_role_distribution(self) -> Dict[str, int]:
        """获取各角色消息数量分布"""
        return {role: len(list(msgs))
                for role, msgs in groupby(sorted(self.messages, key=lambda x: x.role),
                                          key=lambda x: x.role)}

    def clear_messages(self):
        """清空所有消息"""
        self.messages = []


# 使用示例:
if __name__ == "__main__":
    messages_data = {
        "messages": [
            {
                "role": "system",
                "content": "你是一个文件问答助手"
            },
            {
                "role": "user",
                "content": [
                    {
                        "type": "text",
                        "text": "这个文件讲了什么？"
                    },
                    {
                        "type": "file_url",
                        "file_url": {
                            "url": "https://oss.ffire.cc/files/招标文件备案表（第二次）.pdf",
                            "detai": "auto"
                        }
                    }
                ]
            },
            {
                "role": "assistant",
                "content": "好的"
            },
            {
                "role": "user",
                "content": [
                    {
                        "type": "text",
                        "text": "错了 继续回答"
                    }
                ]
            }
        ]
    }

    # 验证数据
    chat_messages = Messages(**messages_data)

    # chat_messages.

    chat_messages.messages[1].content.model_dump()

# 使用示例:
if __name__ == "__main__":
    messages_data = {
        "messages": [
            {
                "role": "system",
                "content": "你是一个文件问答助手"
            },
            {
                "role": "user",
                "content": [
                    {
                        "type": "text",
                        "text": "这个文件讲了什么？"
                    },
                    {
                        "type": "file_url",
                        "file_url": {
                            "url": "https://oss.ffire.cc/files/招标文件备案表（第二次）.pdf",
                            "detai": "auto"
                        }
                    }
                ]
            },
            {
                "role": "assistant",
                "content": "好的"
            },
            {
                "role": "user",
                "content": [
                    {
                        "type": "text",
                        "text": "错了 继续回答"
                    }
                ]
            }
        ]
    }

    # 创建实例
    chat = ChatMessages(**messages_data)

    # 使用示例：
    # 获取最后一条消息
    last_message = chat.get_last_message()
    print("最后一条消息:", last_message)

    # 获取所有用户消息
    user_messages = chat.get_messages_by_role('user')
    print("用户消息:", user_messages)

    # 获取所有文件URL
    file_urls = chat.get_all_file_urls()
    print("文件URLs:", file_urls)

    # 获取所有文本内容
    texts = chat.get_text_content()
    print("文本内容:", texts)

    # 获取对话对
    conversation_pairs = chat.get_conversation_pairs()
    print("对话对:", conversation_pairs)

    # 获取指定索引的消息
    message_at_2 = chat.get_message_at_index(2)
    print("索引2的消息:", message_at_2)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : x
# @Time         : 2023/5/19 14:02
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from qdrant_client import QdrantClient
qdrant = QdrantClient("http://124.221.252.151:6333") # Connect to existing Qdrant instance, for production
qdrant.create_collection('test', vectors_config={})


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : chat
# @Time         : 2025/4/10 16:06
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import os

from meutils.pipe import *
from meutils.io.files_utils import to_base64
from meutils.llm.clients import AsyncOpenAI, zhipuai_client
from meutils.llm.openai_utils import to_openai_params

from meutils.schemas.openai_types import CompletionRequest

from meutils.apis.proxy.kdlapi import get_one_proxy
from openai import APIStatusError


class Completions(object):

    def __init__(
            self,
            base_url: Optional[str] = None,
            api_key: Optional[str] = None,
            http_client: Optional[httpx.AsyncClient] = None
    ):
        self.base_url = base_url
        self.api_key = api_key

        self.client = AsyncOpenAI(base_url=self.base_url, api_key=self.api_key, http_client=http_client)

    async def create(self, request: CompletionRequest):
        # todo:
        # 1. 可思考模型 思考优先
        # request.thinking = {"type": "enabled"}

        if request.thinking and request.model.startswith("doubao-seed") and request.thinking.get("type") == "disabled":
            choices = {
                          "doubao-seed-1-6-250615",
                          "doubao-seed-1-6-vision-250815"
                      } - {request.model}
            request.model = np.random.choice(list(choices))

            logger.debug(request.model)

        elif request.model.startswith("doubao-seed"):
            request.thinking = {"type": "enabled"}
            choices = {
                          "doubao-seed-1-6-250615",
                          "doubao-seed-1-6-vision-250815",
                          "doubao-seed-1-6-thinking-250715"
                      } - {request.model}
            request.model = np.random.choice(list(choices))
            if request.model == "doubao-seed-1-6-thinking-250615":
                request.max_tokens = 16000

        elif request.model.startswith(("doubao-1-5-thinking",)):  # thinking
            request.thinking = None
            choices = {
                          "doubao-1-5-thinking-pro-250415",
                          "doubao-1-5-ui-tars-250428",
                      } - {request.model}
            request.model = np.random.choice(list(choices))

        elif request.model in {"doubao-1-5-thinking-vision-pro-250428"}:  # vision thinking
            choices = {
                          "doubao-1-5-ui-tars-250428",
                      } - {request.model}
            request.model = np.random.choice(list(choices))

        elif request.model.startswith(("doubao-1-5", "doubao-1.5")):  # nothinking
            request.thinking = {"type": "disabled"}

            choices = {
                          "doubao-1-5-pro-32k-250115",
                          "doubao-1-5-pro-256k-250115",
                          "doubao-1-5-pro-32k-character-250715",
                          "doubao-1-5-pro-32k-character-250228",
                          "doubao-1.5-vision-pro-250328",
                          "doubao-1-5-vision-pro-32k-250115",
                          "doubao-1-5-thinking-pro-250415",  # todo 好像没有免费额度
                          # "doubao-1-5-thinking-pro-m-250428",
                          # "doubao-1-5-thinking-vision-pro-250428",
                          "doubao-1-5-ui-tars-250428",
                      } - {request.model}
            request.model = np.random.choice(list(choices))

        elif request.model.startswith(("deepseek-",)):  # deepseek => v3-1
            if request.model.startswith("deepseek-r"):
                request.thinking = {"type": "enabled"}

            choices = {
                          "deepseek-v3-1-250821",
                          "deepseek-v3-1-terminus",
                      } - {request.model}
            request.model = np.random.choice(list(choices))

        ###########################################################################
        # 开启视觉模型
        if not any(i in request.model.lower() for i in ["v-", "vl", "ui", 'vision', 'doubao-seed']) and (
                urls := request.last_urls.get("image_url")):
            # logger.debug(request)
            if request.model.startswith(("gemini",)):

                if urls[-1].startswith("http"):
                    base64_list = await to_base64(urls, content_type="image/png")
                else:
                    base64_list = urls  # 仅支持base64 todo: tokens怎么计算的

                request.messages = [
                    {
                        'role': 'user',
                        'content': [
                            {
                                'type': 'text',
                                'text': request.last_user_content
                            },
                            *[
                                {
                                    'type': 'image_url',
                                    'image_url': {
                                        'url': base64_data
                                    }
                                }
                                for base64_data in base64_list
                            ]

                        ]
                    }
                ]
            else:
                request.model = "glm-4v-flash"
                request.max_tokens = None
                self.client = zhipuai_client

        elif "deepseek-r" in request.model:
            request.separate_reasoning = True  # pp
            # """Error code: 403 - {'code': 403, 'reason': 'NOT_ENOUGH_BALANCE', 'message': 'not enough balance', 'metadata': {}}"""
        ###########################################################################

        data = to_openai_params(request)
        if 'gemini' in request.model:
            data.pop("seed", None)
            data.pop("presence_penalty", None)
            data.pop("frequency_penalty", None)
            data.pop("extra_body", None)

            if "thinking" in request.model:
                data['model'] = data['model'].removesuffix("-thinking")  # 开启思考
                data['reasoning_effort'] = 'low'

        # logger.debug(data)
        try:
            response = await self.client.chat.completions.create(**data)
            return response
        except APIStatusError as e:
            logger.debug(e)
            # logger.debug(e.response.json())
            # logger.debug(e.response.status_code)

            if e.response.status_code > 403 and any(i in self.base_url for i in {"siliconflow", "modelscope"}):
                proxy = await get_one_proxy()
                client = AsyncOpenAI(
                    base_url=self.base_url,
                    api_key=self.api_key,
                    http_client=httpx.AsyncClient(proxy=proxy, timeout=100)
                )
                response = await client.chat.completions.create(**data)
                return response
            raise e


if __name__ == '__main__':
    # 测试 token 1800

    request = CompletionRequest(
        # model="gemini-2.0-flash",
        # model="glm-4-flash",
        # model="deepseek-ai/DeepSeek-V3",
        # model="Qwen/Qwen3-8B1",
        # model="Qwen/Qwen3-8B1",
        # model="doubao-1-5-thinking",
        # model="doubao-1-5",
        model="doubao-seed",
        # model="doubao-seed-1-6-250615",

        thinking={"type": "disabled"},
        messages=[
            {
                "role": "system",
                "content": [
                    {
                        "type": "text",
                        "text": "你是一个数学助手，擅长解决复杂的数学问题。"
                    }
                ]
            },
            # {"role": "user", "content": "你好"},
            {"role": "user", "content": [
                {
                    "type": "text",
                    "text": "解释下" * 1
                },
                # {
                #     "image_url": {
                #         # "detail": "auto",
                #         "url": "https://oss.ffire.cc/files/kling_watermark.png"
                #     },
                #     "type": "image_url"
                # }
            ]}
        ],
        # stream=False,
        # stream=True,

        max_tokens=10,
    )
    # arun(Completions().create(request))
    # d = {
    #     "model": "gemini-2.5-pro-exp-03-25",
    #     "messages": [
    #         {
    #             "content": [
    #                 {
    #                     "text": "The following is an open-ended problem from an International Math competition. Please calculate the answer according to the given requirements and the information provided. Please use LaTeX format to represent the variables and formulas used in the solution process and results. Please end your solution with \"So the final answer is \boxed{multiple answers connected with commas}(unit).\" and give the result explicitly, note that the unit of the answer should not be included in \boxed{}.\nConsider the following system of equations in which all logarithms have base 10:\n\n$$\n\begin{aligned}\n(\log x)(\log y)-3 \log 5 y-\log 8 x & =a \\n(\log y)(\log z)-4 \log 5 y-\log 16 z & =b \\n(\log z)(\log x)-4 \log 8 x-3 \log 625 z & =c\n\end{aligned}\n$$\nIf $a=-4, b=4$, and $c=-18$, solve the system of equations.",
    #                     "type": "text"
    #                 },
    #                 {
    #                     "image_url": {
    #                         "detail": "low",
    #                         "url": "data:image/jpeg;base64,/9j/4AAQSkZJRgABAQAAAQABAAD/2wBDAAgGBgcGBQgHBwcJCQgKDBQNDAsLDBkSEw8UHRofHh0aHBwgJC4nICIsIxwcKDcpLDAxNDQ0Hyc5PTgyPC4zNDL/2wBDAQkJCQwLDBgNDRgyIRwhMjIyMjIyMjIyMjIyMjIyMjIyMjIyMjIyMjIyMjIyMjIyMjIyMjIyMjIyMjIyMjIyMjL/wAARCABAAEADASIAAhEBAxEB/8QAHwAAAQUBAQEBAQEAAAAAAAAAAAECAwQFBgcICQoL/8QAtRAAAgEDAwIEAwUFBAQAAAF9AQIDAAQRBRIhMUEGE1FhByJxFDKBkaEII0KxwRVS0fAkM2JyggkKFhcYGRolJicoKSo0NTY3ODk6Q0RFRkdISUpTVFVWV1hZWmNkZWZnaGlqc3R1dnd4eXqDhIWGh4iJipKTlJWWl5iZmqKjpKWmp6ipqrKztLW2t7i5usLDxMXGx8jJytLT1NXW19jZ2uHi4+Tl5ufo6erx8vP09fb3+Pn6/8QAHwEAAwEBAQEBAQEBAQAAAAAAAAECAwQFBgcICQoL/8QAtREAAgECBAQDBAcFBAQAAQJ3AAECAxEEBSExBhJBUQdhcRMiMoEIFEKRobHBCSMzUvAVYnLRChYkNOEl8RcYGRomJygpKjU2Nzg5OkNERUZHSElKU1RVVldYWVpjZGVmZ2hpanN0dXZ3eHl6goOEhYaHiImKkpOUlZaXmJmaoqOkpaanqKmqsrO0tba3uLm6wsPExcbHyMnK0tPU1dbX2Nna4uPk5ebn6Onq8vP09fb3+Pn6/9oADAMBAAIRAxEAPwD3+iiigAooooAKKKKACiiigAooooAKKKKACiiigAooooAKKKKACiiigAooooAKKKKACiiigAooooAKKKKACiiigD//2Q=="
    #                     },
    #                     "type": "image_url"
    #                 }
    #             ],
    #             "role": "user"
    #         }
    #     ],
    #     "stream": False,
    #     "top_p": 0.7,
    #     "temperature": 0.7,
    #     "n": 1
    # }

    d = {
        "model": "gemini-2.5-pro-exp-03-25",
        # "model": "gemini-2.5-flash-preview-04-17",
        # "model": "gemini-2.5-pro-exp-03-25-thinking",

        "messages": [
            {
                "content": [
                    {
                        "text": "9个8如何加减乘除运直得到1000",
                        "type": "text"
                    },

                ],
                "role": "user"
            }
        ],
        "stream": False,
        "top_p": 0.7,
        "temperature": 0.7,
        "n": 1,
        "reasoning_effort": None
    }
    # api_key = os.getenv("GOOGLE_API_KEY")
    # base_url = os.getenv("GOOGLE_BASE_URL")
    # arun(Completions(api_key=api_key, base_url=base_url).create(request.construct(**d)))

    # request = CompletionRequest(
    #     # model="gemini-2.0-flash",
    #     # model="glm-4-flash",
    #     model="deepseek/deepseek-r1-turbo",
    #
    #     messages=[
    #
    #         {"role": "user", "content": "你好"},
    #
    #     ],
    #     stream=False,
    #     max_tokens=None,
    # )
    # api_key = os.getenv("SILICONFLOW_API_KEY")
    # base_url = "https://api.siliconflow.cn"
    # arun(Completions(api_key=api_key, base_url=base_url).create(request))

    api_key = os.getenv("VOLC_API_KEY")
    base_url = os.getenv("VOLC_BASE_URL")


    async def main():
        async for i in await Completions(api_key=api_key, base_url=base_url).create(request):
            logger.debug(i)


    arun(Completions(api_key=api_key, base_url=base_url).create(request))

    # arun(main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : utils
# @Time         : 2025/3/23 11:06
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *

from openai import AsyncClient


class BaseTextin(object):
    def __init__(self, api_key: Optional[str]):
        # https://www.textin.com/console/dashboard/setting
        app_id, secret_code = api_key or os.getenv("TEXTIN_API_KEY").split("|")

        self.base_url="https://api.textin.com/ai/service/v1"
        self.headers = {
            'x-ti-app-id': app_id,
            'x-ti-secret-code': secret_code
        }
        # self.client = AsyncClient(
        #     base_url="https://api.textin.com/ai/service/v1",  # /image/watermark_remove
        #     default_headers=headers
        # )


    def image_watermark_remove(self, image):
        """

        :param image: text/plain or application/octet-stream
        :return:
        """
        pass


if __name__ == "__main__":
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : __init__.py
# @Time         : 2021/3/9 2:12 下午
# @Author       : yuanjie
# @WeChat       : 313303303
# @Software     : PyCharm
# @Description  :



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : images
# @Time         : 2025/8/11 16:04
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 


from meutils.pipe import *
from meutils.db.redis_db import redis_aclient
from meutils.io.files_utils import to_bytes

from meutils.apis.utils import make_request_httpx
from meutils.llm.clients import AsyncClient
from meutils.llm.openai_utils import to_openai_params

from meutils.schemas.gitee_types import FEISHU_URL, BASE_URL
from meutils.schemas.image_types import ImageRequest, ImagesResponse

from meutils.config_utils.lark_utils import get_next_token_for_polling


async def generate(request: ImageRequest, api_key: Optional[str] = None):
    api_key = api_key or await get_next_token_for_polling(FEISHU_URL)

    client = AsyncClient(base_url=BASE_URL, api_key=api_key)

    data = {
        **request.model_dump(exclude_none=True, exclude={"extra_fields", "aspect_ratio"}),
        **(request.extra_fields or {})
    }

    if request.model.endswith("-lora"):
        data["model"] = data["model"].removesuffix("-lora")
        if "Qwen-Image" in data["model"]:
            data = {
                **{
                    "cfg_scale": 1,
                    "num_inference_steps": 8,
                    "lora_weights": [
                        {
                            "url": "https://gitee.com/realhugh/materials/raw/master/Qwen-Image-Lightning-8steps-V1.1.safetensors",
                            "weight": 1
                        }
                    ]
                },
                **data,
            }

    # logger.debug(bjson(data))

    data = to_openai_params(ImageRequest(**data))
    logger.debug(bjson(data))

    response = await client.images.generate(**data)
    return response


if __name__ == '__main__':
    data = {
        "prompt": "A robot sitting on open grassland, painting on a canvas.",
        "model": "FLUX_1-Krea-dev",
        "size": "1024x576",
        "num_inference_steps": 28,
        "guidance_scale": 4.5,
        "seed": 42
    }
    data = {
        "prompt": "一幅精致细腻的工笔画，画面中心是一株蓬勃生长的红色牡丹，花朵繁茂，既有盛开的硕大花瓣，也有含苞待放的花蕾，层次丰富，色彩艳丽而不失典雅。牡丹枝叶舒展，叶片浓绿饱满，脉络清晰可见，与红花相映成趣。一只蓝紫色蝴蝶仿佛被画中花朵吸引，停驻在画面中央的一朵盛开牡丹上，流连忘返，蝶翼轻展，细节逼真，仿佛随时会随风飞舞。整幅画作笔触工整严谨，色彩浓郁鲜明，展现出中国传统工笔画的精妙与神韵，画面充满生机与灵动之感。",
        "model": "Qwen-Image-lora",
        "size": "1024x1024",
        "num_inference_steps": 30,
        "cfg_scale": 4
    }
    arun(generate(ImageRequest(**data)))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : videos
# @Time         : 2024/12/12 08:53
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from meutils.caches import rcache
from meutils.io.files_utils import to_url, to_url_fal, to_base64
from meutils.llm.check_utils import check_token_for_siliconflow
from meutils.schemas.task_types import TaskResponse
from meutils.schemas.siliconflow_types import FEISHU_URL, FEISHU_URL_FREE, BASE_URL, VideoRequest
from meutils.config_utils.lark_utils import get_next_token_for_polling

from openai import OpenAI, AsyncOpenAI

check_token = partial(check_token_for_siliconflow, threshold=0.01)

"""

tencent/HunyuanVideo-HD

Wan-AI/Wan2.1-T2V-14B
Wan-AI/Wan2.1-T2V-14B-Turbo

Wan-AI/Wan2.1-I2V-14B-720P
Wan-AI/Wan2.1-I2V-14B-720P-Turbo


16:9 👉 1280×720
9:16 👉 720×1280
1:1 👉 960×960

"""


# @rcache(ttl=0.5 * 24 * 3600, serializer="pickle")
async def create_task(request: VideoRequest, token: Optional[str] = None):
    token = token or await get_next_token_for_polling(FEISHU_URL_FREE, check_token=check_token, from_redis=True)

    if 'Wan-AI' in request.model:
        request.model = "Wan-AI/Wan2.1-T2V-14B-720P-Turbo"

    if request.image:
        # request.image = await to_base64(request.image)
        request.model = request.model.replace("-T2V-", "-I2V-")

    payload = request.model_dump(exclude_none=True)

    logger.debug(payload)

    client = AsyncOpenAI(
        base_url=BASE_URL,
        api_key=token
    )

    response = await client.post("/video/submit", body=payload, cast_to=object)
    task_id = response.get('requestId')

    return TaskResponse(task_id=task_id, system_fingerprint=token)


async def get_task(task_id, token: str):
    client = AsyncOpenAI(
        base_url=BASE_URL,
        api_key=token
    )
    payload = {"requestId": task_id}
    response = await client.post(f"/video/status", cast_to=object, body=payload)
    logger.debug(response)

    data = response.get("results") or {}

    # for video in data.get("videos", []):
    #     video["url"] = await to_url_fal(video.get("url"), content_type="video/mp4")  # 异步执行

    return TaskResponse(
        task_id=task_id,
        data=data,
        status=response.get("status"),
        message=response.get("reason"),
    )


if __name__ == '__main__':
    token = None
    token = "sk-rfuayacpsnrcikpgwrotzfcnpzhsrgqnfjdgnihckbhhscgw"

    request = VideoRequest(
        model="x",
        prompt="这个女人笑起来 ",
        image='https://oss.ffire.cc/files/kling_watermark.png'  # 1148f2e4-0a62-4208-84de-0bf2c88f740d
    )

    # r = arun(create_task(request))

    # tokens_ = arun(check_token_for_siliconflow(tokens, threshold=0.01))

    # arun(create_task(request, token=token))

    arun(get_task("gkx1e8fejgod", token))
    # arun(get_task("c716a328-438e-4612-aff2-a669034499cb", token))
    # arun(get_task("1148f2e4-0a62-4208-84de-0bf2c88f740d", token))

    # token = "sk-oeptckzkhfzeidbtsqvbrvyrfdtyaaehubfwsxjytszbgohd"
    # arun(get_task(r.task_id, r.system_fingerprint))

    # arun(get_task("c6zxjtpsyywj", "sk-lavdsoaybczrfygqdbbqmdgmdktvdztfvwnfmwnbusewjkwb"))

    # arun(create_task(VideoRequest(model="tencent/HunyuanVideo", prompt="a dog in the forest."), token=token))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : models
# @Time         : 2024/11/18 13:35
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *

BAICHUAN = {
    'baichuan4-air': 0.49,

    'baichuan4-turbo': 7.5,
    'baichuan4': 50,
    'baichuan3-turbo': 6,
    'baichuan3-turbo-128k': 12,
    'baichuan2-turbo': 4,
}

if __name__ == '__main__':
    print(BAICHUAN | xjoin(','))





#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : eidt
# @Time         : 2025/1/7 13:27
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *

from meutils.pipe import *
from meutils.io.files_utils import to_url_fal

from openai import OpenAI

client = OpenAI(
    # base_url="https://ai.gitee.com/v1",
    # api_key="WPCSA3ZYD8KBQQ2ZKTAPVUA059J2Q47TLWGB2ZMQ",
    # default_headers={"X-Failover-Enabled": "true", "X-Package": "1910"},

)

"https://api.aiguoguo199.com/v1/images/generations"
# client = OpenAI(
#     base_url="https://ai.gitee.com/v1",
#     api_key="WPCSA3ZYD8KBQQ2ZKTAPVUA059J2Q47TLWGB2ZMQ",
#     default_headers={"X-Failover-Enabled": "true", "X-Package": "1910"},
# )
response = client.images.edit(
    # model="Kolors",
    model="gemini-2.0-flash-exp-image-generation",
    prompt="笑起来",
    size="1024x1024",
    # extra_body={
    #     "steps": 25,
    #     "guidance_scale": 6,
    # },

    # mask=open("test.png", "rb"),
    image=open("test.png", "rb"),
)

if __name__ == '__main__':
    arun(to_url_fal(response.data[0].b64_json, content_type="image/png"))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : tryblend_types
# @Time         : 2024/9/4 08:54
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 


BASE_URL = "https://www.tryblend.ai/"
FEISHU_URL = "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=hxVlQw"
FEISHU_URL_VIP = "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=LYcY2c"

GPT_4O = [{"requestId":"4aee81b2-2379-4bd2-ae7a-99c9c6867c86","selectedModel":{"id":"openai:gpt-4o","active":True,"name":"GPT 4o","provider":"openai","maker":"openai","makerHumanName":"OpenAI","fallbackIcon":None,"providerModelId":"gpt-4o","info":{"description":"GPT-4o is OpenAI's most advanced multimodal model that’s faster and cheaper than GPT-4 Turbo with stronger vision capabilities.","fundationModel":"GPT-4","fundationModelMaker":"OpenAI","releaseDate":"2024","knowledgeCutoff":"October, 2023","context":"128000","predictionTimeSecond":None,"usableAsAssistant":True,"pricing":{"inputImagePrice":0.003,"inputPrice":5,"outputPrice":15,"inputPriceUnit":"1m tokens","outputPriceUnit":"1m tokens"},"inputType":["text","image"],"outputType":["text"],"type":["text","vision"],"keywords":["trending","vision"],"website":"https://openai.com/research/gpt-4","fallbackModel":None},"parameters":[{"name":"prompt","description":"The prompt to use for the request","parameterType":"directInput","defaultValue":""},{"name":"file","description":"Files to use as complimentary information for the prompt.","parameterType":"directInput","defaultValue":[],"fileOptions":[{"fileExtension":[".jpg",".jpeg",".webp",".png",".gif"],"fileExtensionName":"image","maxFileSize":20,"minFileCount":0,"maxFileCount":20},{"fileExtension":[".c",".cpp",".cs",".java",".py",".rb",".js",".ts",".tsx",".jsx",".php",".go",".swift",".kt",".scala",".rs",".lua",".pl",".sh",".bash",".ps1",".html",".css",".scss",".less",".json",".xml",".yaml",".yml",".md",".markdown",".txt",".rtf",".tex",".log",".xlsx",".pdf",".csv",".tsv",".ini",".cfg",".conf",".toml",".sql",".r",".m",".f",".f90",".vb",".bas",".ps",".asm"],"fileExtensionName":"text","maxFileSize":5,"minFileCount":0,"maxFileCount":10}]}]},"messages":[{"role":"user","content":[{"type":"text","text":"1"}]}],"sessionId":"bb06bb82-708e-4c98-b9db-3771d654195e","userMessageId":"4b1a58ac-55f9-4655-ab3e-1d7cff788b70","assistantMessageId":"4c7eb0ce-69b3-42d9-bf7c-3692f3c04e47"}]
GPT_4O_MINI = [{"requestId":"9146047d-387a-4894-b64a-879401a0bed0","selectedModel":{"id":"openai:gpt-4o-mini","active":True,"name":"GPT 4o Mini","provider":"openai","maker":"openai","makerHumanName":"OpenAI","fallbackIcon":None,"providerModelId":"gpt-4o-mini","info":{"description":"GPT-4o mini is OpenAI's most cost-efficient small model that’s smarter and cheaper than GPT-3.5 Turbo, and has vision capabilities.","fundationModel":"GPT-4","fundationModelMaker":"OpenAI","releaseDate":"2024","knowledgeCutoff":"October, 2023","context":"128000","predictionTimeSecond":None,"usableAsAssistant":True,"pricing":{"inputImagePrice":0.003,"inputPrice":0.15,"outputPrice":0.6,"inputPriceUnit":"1m tokens","outputPriceUnit":"1m tokens"},"inputType":["text","image"],"outputType":["text"],"type":["text","vision"],"keywords":["free","vision"],"website":"https://openai.com/research/gpt-4","fallbackModel":None},"parameters":[{"name":"prompt","description":"The prompt to use for the request","parameterType":"directInput","defaultValue":""},{"name":"file","description":"Files to use as complimentary information for the prompt.","parameterType":"directInput","defaultValue":[],"fileOptions":[{"fileExtension":[".jpg",".jpeg",".webp",".png",".gif"],"fileExtensionName":"image","maxFileSize":5,"minFileCount":0,"maxFileCount":20},{"fileExtension":[".c",".cpp",".cs",".java",".py",".rb",".js",".ts",".tsx",".jsx",".php",".go",".swift",".kt",".scala",".rs",".lua",".pl",".sh",".bash",".ps1",".html",".css",".scss",".less",".json",".xml",".yaml",".yml",".md",".markdown",".txt",".rtf",".tex",".log",".xlsx",".pdf",".csv",".tsv",".ini",".cfg",".conf",".toml",".sql",".r",".m",".f",".f90",".vb",".bas",".ps",".asm"],"fileExtensionName":"text","maxFileSize":5,"minFileCount":0,"maxFileCount":10}]}]},"messages":[{"role":"user","content":[{"type":"text","text":"1"}]}],"sessionId":"7f27c6b6-1e98-4bb4-9e90-6d68d759632e","userMessageId":"e63eadcf-6715-40a5-805f-ce9442c4518c","assistantMessageId":"ab0971c4-03fa-4c96-9932-b4019d1ded46"}]
CLAUDE_3_HAIKU = [{"requestId":"86ce06bd-f76c-4cc4-91af-062907091af9","selectedModel":{"id":"openrouter:anthropic/claude-3-haiku","active":True,"name":"Claude 3 Haiku","provider":"openrouter","maker":"anthropic","makerHumanName":"Anthropic","fallbackIcon":None,"providerModelId":"anthropic/claude-3-haiku","info":{"description":"Claude 3 Haiku is Anthropic's fastest and most compact model for near-instant responsiveness. Quick and accurate targeted performance.","fundationModel":None,"fundationModelMaker":None,"releaseDate":"March 4, 2024","knowledgeCutoff":None,"context":"200000","predictionTimeSecond":None,"usableAsAssistant":False,"pricing":{"inputImagePrice":0.0004,"inputPrice":0.25,"outputPrice":1.25,"inputPriceUnit":"1m tokens","outputPriceUnit":"1m tokens"},"inputType":["text","image"],"outputType":["text"],"type":["text","vision"],"keywords":["free","vision"],"website":"https://www.anthropic.com/news/claude-3-family","fallbackModel":None},"parameters":[{"name":"prompt","description":"The prompt to use for the request","parameterType":"directInput","defaultValue":""},{"name":"file","description":"Files to use as complimentary information for the prompt.","parameterType":"directInput","defaultValue":[],"fileOptions":[{"fileExtension":[".jpg",".jpeg",".webp",".png",".gif"],"fileExtensionName":"image","maxFileSize":5,"minFileCount":0,"maxFileCount":20},{"fileExtension":[".c",".cpp",".cs",".java",".py",".rb",".js",".ts",".tsx",".jsx",".php",".go",".swift",".kt",".scala",".rs",".lua",".pl",".sh",".bash",".ps1",".html",".css",".scss",".less",".json",".xml",".yaml",".yml",".md",".markdown",".txt",".rtf",".tex",".log",".xlsx",".pdf",".csv",".tsv",".ini",".cfg",".conf",".toml",".sql",".r",".m",".f",".f90",".vb",".bas",".ps",".asm"],"fileExtensionName":"text","maxFileSize":5,"minFileCount":0,"maxFileCount":10}]}]},"messages":[{"role":"user","content":[{"type":"text","text":"1"}]},{"role":"assistant","content":[{"type":"text","text":"Hello! It looks like you entered \"1.\" How can I assist you today? If you have a question or need information, feel free to let me know!"}]},{"role":"user","content":[{"type":"text","text":"1"}]}],"sessionId":"bb06bb82-708e-4c98-b9db-3771d654195e","userMessageId":"313e734a-fd15-404a-8132-92548aa783b5","assistantMessageId":"5610ede0-3da9-4c0e-bfab-c83fb282312f"}]
CLAUDE_3_SONNET = [{"requestId":"00e9b684-29c1-42b4-b2c5-2d7eb6c0be25","selectedModel":{"id":"openrouter:anthropic/claude-3-sonnet","active":True,"name":"Claude 3 Sonnet","provider":"openrouter","maker":"anthropic","makerHumanName":"Anthropic","fallbackIcon":None,"providerModelId":"anthropic/claude-3-sonnet","info":{"description":"Claude 3 Sonnet is an ideal balance of intelligence and speed for enterprise workloads. Maximum utility at a lower price, dependable, balanced for scaled deployments.","fundationModel":None,"fundationModelMaker":None,"releaseDate":"March 4, 2024","knowledgeCutoff":None,"context":"200000","predictionTimeSecond":None,"usableAsAssistant":False,"pricing":{"inputImagePrice":0.0048,"inputPrice":3,"outputPrice":15,"inputPriceUnit":"1m tokens","outputPriceUnit":"1m tokens"},"inputType":["text","image"],"outputType":["text"],"type":["text","vision"],"keywords":["vision"],"website":"https://www.anthropic.com/news/claude-3-family","fallbackModel":None},"parameters":[{"name":"prompt","description":"The prompt to use for the request","parameterType":"directInput","defaultValue":""},{"name":"file","description":"Files to use as complimentary information for the prompt.","parameterType":"directInput","defaultValue":[],"fileOptions":[{"fileExtension":[".jpg",".jpeg",".webp",".png",".gif"],"fileExtensionName":"image","maxFileSize":5,"minFileCount":0,"maxFileCount":20},{"fileExtension":[".c",".cpp",".cs",".java",".py",".rb",".js",".ts",".tsx",".jsx",".php",".go",".swift",".kt",".scala",".rs",".lua",".pl",".sh",".bash",".ps1",".html",".css",".scss",".less",".json",".xml",".yaml",".yml",".md",".markdown",".txt",".rtf",".tex",".log",".xlsx",".pdf",".csv",".tsv",".ini",".cfg",".conf",".toml",".sql",".r",".m",".f",".f90",".vb",".bas",".ps",".asm"],"fileExtensionName":"text","maxFileSize":5,"minFileCount":0,"maxFileCount":10}]}]},"messages":[{"role":"user","content":[{"type":"text","text":"1"}]}],"sessionId":"ea188159-85ce-49da-8bf3-c8364f497e1a","userMessageId":"ab21e0dd-993d-44f0-b7be-a80baf2460f1","assistantMessageId":"cf5bbf51-1c8d-4465-a35f-0cf5ced9e7ef"}]
CLAUDE_3_OPUS = [{"requestId":"ee6e9a61-4546-499a-aa00-03115a088206","selectedModel":{"id":"openrouter:anthropic/claude-3-opus","active":True,"name":"Claude 3 Opus","provider":"openrouter","maker":"anthropic","makerHumanName":"Anthropic","fallbackIcon":None,"providerModelId":"anthropic/claude-3-opus","info":{"description":"Claude 3 Opus is Anthropic's most powerful model for highly complex tasks. It boasts top-level performance, intelligence, fluency, and understanding. It beats GPT-4 on most benchmarks","fundationModel":None,"fundationModelMaker":None,"releaseDate":"March 4, 2024","knowledgeCutoff":None,"context":"200000","predictionTimeSecond":None,"usableAsAssistant":False,"pricing":{"inputImagePrice":0.024,"inputPrice":15,"outputPrice":75,"inputPriceUnit":"1m tokens","outputPriceUnit":"1m tokens"},"inputType":["text","image"],"outputType":["text"],"type":["text","vision"],"keywords":["vision"],"website":"https://www.anthropic.com/news/claude-3-family","fallbackModel":None},"parameters":[{"name":"prompt","description":"The prompt to use for the request","parameterType":"directInput","defaultValue":""},{"name":"file","description":"Files to use as complimentary information for the prompt.","parameterType":"directInput","defaultValue":[],"fileOptions":[{"fileExtension":[".jpg",".jpeg",".webp",".png",".gif"],"fileExtensionName":"image","maxFileSize":5,"minFileCount":0,"maxFileCount":20},{"fileExtension":[".c",".cpp",".cs",".java",".py",".rb",".js",".ts",".tsx",".jsx",".php",".go",".swift",".kt",".scala",".rs",".lua",".pl",".sh",".bash",".ps1",".html",".css",".scss",".less",".json",".xml",".yaml",".yml",".md",".markdown",".txt",".rtf",".tex",".log",".xlsx",".pdf",".csv",".tsv",".ini",".cfg",".conf",".toml",".sql",".r",".m",".f",".f90",".vb",".bas",".ps",".asm"],"fileExtensionName":"text","maxFileSize":5,"minFileCount":0,"maxFileCount":10}]}]},"messages":[{"role":"user","content":[{"type":"text","text":"1"}]}],"sessionId":"f112a636-bda4-47e1-9e35-8fea5a12cbf3","userMessageId":"1a80572b-c7cb-4407-a301-ba8471cdc23c","assistantMessageId":"d2e275af-4648-4d60-a292-38766f5a0425"}]
CLAUDE_35_SONNET = [{"requestId":"e1da0fc4-ec34-4784-a336-0dce0f7152d1","selectedModel":{"id":"openrouter:anthropic/claude-3.5-sonnet","active":True,"name":"Claude 3.5 Sonnet","provider":"openrouter","maker":"anthropic","makerHumanName":"Anthropic","fallbackIcon":None,"providerModelId":"anthropic/claude-3.5-sonnet","info":{"description":"Claude 3.5 Sonnet delivers better-than-Opus capabilities, faster-than-Sonnet speeds, at the same Sonnet prices. Sonnet is particularly good at: Coding: Autonomously writes, edits, and runs code with reasoning and troubleshooting Data science: Augments human data science expertise; navigates unstructured data while using multiple tools for insights Visual processing: excelling at interpreting charts, graphs, and images, accurately transcribing text to derive insights beyond just the text alone Agentic tasks: exceptional tool use, making it great at agentic tasks (i.e. complex, multi-step problem solving tasks that require engaging with other systems)","fundationModel":None,"fundationModelMaker":None,"releaseDate":"2024","knowledgeCutoff":"2024","context":"200000","predictionTimeSecond":None,"usableAsAssistant":False,"pricing":{"inputImagePrice":0.0048,"inputPrice":3,"outputPrice":15,"inputPriceUnit":"1m tokens","outputPriceUnit":"1m tokens"},"inputType":["text","image"],"outputType":["text"],"type":["text","vision"],"keywords":["trending","vision"],"website":"https://www.anthropic.com/news/claude-3-family","fallbackModel":None},"parameters":[{"name":"prompt","description":"The prompt to use for the request","parameterType":"directInput","defaultValue":""},{"name":"file","description":"Files to use as complimentary information for the prompt.","parameterType":"directInput","defaultValue":[],"fileOptions":[{"fileExtension":[".jpg",".jpeg",".webp",".png",".gif"],"fileExtensionName":"image","maxFileSize":5,"minFileCount":0,"maxFileCount":20},{"fileExtension":[".c",".cpp",".cs",".java",".py",".rb",".js",".ts",".tsx",".jsx",".php",".go",".swift",".kt",".scala",".rs",".lua",".pl",".sh",".bash",".ps1",".html",".css",".scss",".less",".json",".xml",".yaml",".yml",".md",".markdown",".txt",".rtf",".tex",".log",".xlsx",".pdf",".csv",".tsv",".ini",".cfg",".conf",".toml",".sql",".r",".m",".f",".f90",".vb",".bas",".ps",".asm"],"fileExtensionName":"text","maxFileSize":5,"minFileCount":0,"maxFileCount":10}]}]},"messages":[{"role":"user","content":[{"type":"text","text":"1"}]}],"sessionId":"ea24b58e-6132-45ab-a46e-b8ff7bd235e2","userMessageId":"74f40c8a-bc94-4fb4-96cc-4b118309cf61","assistantMessageId":"79f48170-71ad-45bd-b895-a1dc4a17ffa3"}]
PERPLEXITY_SONAR_SMALL = [{"requestId":"a3338192-e46f-4ae3-83b2-8e428935c8ea","selectedModel":{"id":"perplexity/llama-3.1-sonar-small-128k-online","active":True,"name":"Perplexity Sonar Small","provider":"perplexity","maker":"perplexity","makerHumanName":"Perplexity","fallbackIcon":None,"providerModelId":"llama-3.1-sonar-small-128k-online","info":{"description":"The smaller, internet-connected chat model by Perplexity Labs, based on Llama 3.1. The online models are focused on delivering helpful, up-to-date, and factual responses.","fundationModel":"LLama 3 8B","fundationModelMaker":"Meta","releaseDate":"August 2024","knowledgeCutoff":"Today","context":"128000","predictionTimeSecond":None,"usableAsAssistant":False,"pricing":{"inputImagePrice":None,"inputPrice":0.2,"outputPrice":0.2,"inputPriceUnit":"request","outputPriceUnit":"1m tokens"},"inputType":["text"],"outputType":["text"],"type":["text"],"keywords":["free","online search"],"website":"https://blog.perplexity.ai/blog/introducing-pplx-online-llms","fallbackModel":None},"parameters":[{"name":"prompt","description":"The prompt to use for the request","parameterType":"directInput","defaultValue":""},{"name":"file","description":"Files to use as complimentary information for the prompt.","parameterType":"directInput","defaultValue":[],"fileOptions":[{"fileExtension":[".c",".cpp",".cs",".java",".py",".rb",".js",".ts",".tsx",".jsx",".php",".go",".swift",".kt",".scala",".rs",".lua",".pl",".sh",".bash",".ps1",".html",".css",".scss",".less",".json",".xml",".yaml",".yml",".md",".markdown",".txt",".rtf",".tex",".log",".xlsx",".pdf",".csv",".tsv",".ini",".cfg",".conf",".toml",".sql",".r",".m",".f",".f90",".vb",".bas",".ps",".asm"],"fileExtensionName":"text","maxFileSize":5,"minFileCount":0,"maxFileCount":10}]}]},"messages":[{"role":"user","content":[{"type":"text","text":"1"}]},{"role":"assistant","content":[{"type":"text","text":"It appears you might have entered a number without providing context. Here are a few possible interpretations:\n\n1. **Numerical Value**: If you simply entered \"1\" as a number, it could represent various things depending on the context, such as a quantity, a ranking, or a specific item in a list.\n2. **Question or Request**: If you are asking for information or assistance related to the number \"1,\" please provide more details so I can help you better.\n3. **List or Sequence**: If you are starting a list or sequence, \"1\" would be the first item.\n\nTo provide a more accurate response, could you please clarify what you are looking for?"}]},{"role":"user","content":[{"type":"text","text":"1"}]}],"sessionId":"430642e3-05e7-42cf-ac09-860202456dd5","userMessageId":"89a39d94-2d7b-4c89-b588-721d93a18bb6","assistantMessageId":"085f5380-5054-4260-be0f-2333486156af"}]
PERPLEXITY_SONAR_LARGE = [{"requestId":"ff8e920c-6ec3-4d26-8e47-9c0357bd38d4","selectedModel":{"id":"perplexity/llama-3.1-sonar-large-128k-online","active":True,"name":"Perplexity Sonar Large","provider":"perplexity","maker":"perplexity","makerHumanName":"Perplexity","fallbackIcon":None,"providerModelId":"llama-3.1-sonar-large-128k-online","info":{"description":"The larger, internet-connected chat model by Perplexity Labs, based on Llama 3.1. The online models are focused on delivering helpful, up-to-date, and factual responses.","fundationModel":"LLama 3 70B","fundationModelMaker":"Meta","releaseDate":"August 2024","knowledgeCutoff":"Today","context":"128000","predictionTimeSecond":None,"usableAsAssistant":False,"pricing":{"inputImagePrice":None,"inputPrice":1,"outputPrice":1,"inputPriceUnit":"request","outputPriceUnit":"1m tokens"},"inputType":["text"],"outputType":["text"],"type":["text"],"keywords":["online search"],"website":"https://blog.perplexity.ai/blog/introducing-pplx-online-llms","fallbackModel":None},"parameters":[{"name":"prompt","description":"The prompt to use for the request","parameterType":"directInput","defaultValue":""},{"name":"file","description":"Files to use as complimentary information for the prompt.","parameterType":"directInput","defaultValue":[],"fileOptions":[{"fileExtension":[".c",".cpp",".cs",".java",".py",".rb",".js",".ts",".tsx",".jsx",".php",".go",".swift",".kt",".scala",".rs",".lua",".pl",".sh",".bash",".ps1",".html",".css",".scss",".less",".json",".xml",".yaml",".yml",".md",".markdown",".txt",".rtf",".tex",".log",".xlsx",".pdf",".csv",".tsv",".ini",".cfg",".conf",".toml",".sql",".r",".m",".f",".f90",".vb",".bas",".ps",".asm"],"fileExtensionName":"text","maxFileSize":5,"minFileCount":0,"maxFileCount":10}]}]},"messages":[{"role":"user","content":[{"type":"text","text":"1"}]}],"sessionId":"430642e3-05e7-42cf-ac09-860202456dd5","userMessageId":"09d1272e-935d-4427-ac79-a206c1ed03bb","assistantMessageId":"65378f38-082c-4ab6-ab40-8a39c741e741"}]
PERPLEXITY_SONAR_HUGE = [{"requestId":"daef312d-818d-4e90-827e-7dd388df2464","selectedModel":{"id":"perplexity/llama-3.1-sonar-huge-128k-online","active":True,"name":"Perplexity Sonar Huge","provider":"perplexity","maker":"perplexity","makerHumanName":"Perplexity","fallbackIcon":None,"providerModelId":"llama-3.1-sonar-huge-128k-online","info":{"description":"The larger, internet-connected chat model by Perplexity Labs, based on Llama 3.1. The online models are focused on delivering helpful, up-to-date, and factual responses.","fundationModel":"LLama 3 405B","fundationModelMaker":"Meta","releaseDate":"August 2024","knowledgeCutoff":"Today","context":"128000","predictionTimeSecond":None,"usableAsAssistant":False,"pricing":{"inputImagePrice":None,"inputPrice":5,"outputPrice":5,"inputPriceUnit":"request","outputPriceUnit":"1m tokens"},"inputType":["text"],"outputType":["text"],"type":["text"],"keywords":["trending","online search"],"website":"https://blog.perplexity.ai/blog/introducing-pplx-online-llms","fallbackModel":None},"parameters":[{"name":"prompt","description":"The prompt to use for the request","parameterType":"directInput","defaultValue":""},{"name":"file","description":"Files to use as complimentary information for the prompt.","parameterType":"directInput","defaultValue":[],"fileOptions":[{"fileExtension":[".c",".cpp",".cs",".java",".py",".rb",".js",".ts",".tsx",".jsx",".php",".go",".swift",".kt",".scala",".rs",".lua",".pl",".sh",".bash",".ps1",".html",".css",".scss",".less",".json",".xml",".yaml",".yml",".md",".markdown",".txt",".rtf",".tex",".log",".xlsx",".pdf",".csv",".tsv",".ini",".cfg",".conf",".toml",".sql",".r",".m",".f",".f90",".vb",".bas",".ps",".asm"],"fileExtensionName":"text","maxFileSize":5,"minFileCount":0,"maxFileCount":10}]}]},"messages":[{"role":"user","content":[{"type":"text","text":"11"}]}],"sessionId":"b9cc2133-5da9-4e28-934d-21877246a830","userMessageId":"af132989-38ce-4375-9b1c-512c55d28fba","assistantMessageId":"617bbeb1-eb61-4e85-b4c9-c85f614ceca6"}]
# 画画
FLUX_SCHNELL = {"userMessage":{"id":"81439612-aa86-43a3-a127-6fa2a89b7a37","parentId":"db331b85-ce87-4950-9574-baef444511e6","role":"user","content":"a dog","createdAt":"2024-09-06T06:24:55.571Z","modelId":"falai:fal-ai/flux/schnell","info":{"type":"userTextMessage","isSaved":True},"status":{"isLoading":False},"files":[],"parameters":{}},"selectedModel":{"id":"falai:fal-ai/flux/schnell","active":True,"name":"Flux.1 Schnell","provider":"falai","maker":"black forest labs","makerHumanName":"Black Forest Labs","fallbackIcon":None,"providerModelId":"fal-ai/flux/schnell","info":{"description":"The smallest, cheapest and fastest model of the suite of text-to-image models that define a new state-of-the-art in image detail, prompt adherence, style diversity, and scene complexity for text-to-image synthesis.","fundationModel":None,"fundationModelMaker":None,"releaseDate":"August 1, 2024","knowledgeCutoff":None,"context":None,"predictionTimeSecond":None,"usableAsAssistant":False,"pricing":{"inputImagePrice":0,"inputPrice":0,"outputPrice":0.003,"inputPriceUnit":"image","outputPriceUnit":"megapixel"},"inputType":["text"],"outputType":["image"],"type":["image"],"keywords":["free"],"website":"https://blackforestlabs.ai/announcing-black-forest-labs/?ref=blog.fal.ai","fallbackModel":None},"parameters":[{"name":"prompt","description":"The prompt to use for the request","parameterType":"directInput","defaultValue":""},{"name":"file","description":"Files to use as complimentary information for the prompt.","parameterType":"directInput","defaultValue":[],"fileOptions":[{"fileExtension":[".c",".cpp",".cs",".java",".py",".rb",".js",".ts",".tsx",".jsx",".php",".go",".swift",".kt",".scala",".rs",".lua",".pl",".sh",".bash",".ps1",".html",".css",".scss",".less",".json",".xml",".yaml",".yml",".md",".markdown",".txt",".rtf",".tex",".log",".xlsx",".pdf",".csv",".tsv",".ini",".cfg",".conf",".toml",".sql",".r",".m",".f",".f90",".vb",".bas",".ps",".asm"],"fileExtensionName":"text","maxFileSize":5,"minFileCount":0,"maxFileCount":10}]}]},"sessionId":"8f8500aa-0c8c-411c-b27f-0d9febd7c498","userMessageId":"81439612-aa86-43a3-a127-6fa2a89b7a37","assistantMessageId":"5a060fc3-f9d5-4844-baf3-ae5d8715d0b4","requestId":"61a762e2-4781-4c14-bcb0-7cd481e00820"}
FLUX_PRO = {"userMessage":{"id":"67d8d521-955e-4664-b07b-3c4ecf82c214","parentId":None,"role":"user","content":"11","createdAt":"2024-09-06T06:31:31.047Z","modelId":"falai:fal-ai/flux/pro","info":{"type":"userTextMessage","isSaved":True},"status":{"isLoading":False},"files":[],"parameters":{}},"selectedModel":{"id":"falai:fal-ai/flux/pro","active":True,"name":"Flux.1 Pro","provider":"falai","maker":"black forest labs","makerHumanName":"Black Forest Labs","fallbackIcon":None,"providerModelId":"fal-ai/flux-pro","info":{"description":"The largest model of the suite of text-to-image models that define a new state-of-the-art in image detail, prompt adherence, style diversity, and scene complexity for text-to-image synthesis.","fundationModel":None,"fundationModelMaker":None,"releaseDate":"August 1, 2024","knowledgeCutoff":None,"context":None,"predictionTimeSecond":None,"usableAsAssistant":False,"pricing":{"inputImagePrice":0,"inputPrice":0,"outputPrice":0.05,"inputPriceUnit":"image","outputPriceUnit":"megapixel"},"inputType":["text"],"outputType":["image"],"type":["image"],"keywords":["trending"],"website":"https://blackforestlabs.ai/announcing-black-forest-labs/?ref=blog.fal.ai","fallbackModel":None},"parameters":[{"name":"prompt","description":"The prompt to use for the request","parameterType":"directInput","defaultValue":""},{"name":"file","description":"Files to use as complimentary information for the prompt.","parameterType":"directInput","defaultValue":[],"fileOptions":[{"fileExtension":[".c",".cpp",".cs",".java",".py",".rb",".js",".ts",".tsx",".jsx",".php",".go",".swift",".kt",".scala",".rs",".lua",".pl",".sh",".bash",".ps1",".html",".css",".scss",".less",".json",".xml",".yaml",".yml",".md",".markdown",".txt",".rtf",".tex",".log",".xlsx",".pdf",".csv",".tsv",".ini",".cfg",".conf",".toml",".sql",".r",".m",".f",".f90",".vb",".bas",".ps",".asm"],"fileExtensionName":"text","maxFileSize":5,"minFileCount":0,"maxFileCount":10}]}]},"sessionId":"c1121f53-e40c-4b9e-9321-d54bfe7af39e","userMessageId":"67d8d521-955e-4664-b07b-3c4ecf82c214","assistantMessageId":"02aee5af-474c-496b-b63d-79e86e540ef6","requestId":"7a10bc5c-90c4-494d-9834-17dbab6c3661"}
DALL_E_3 = {"userMessage":{"id":"1753c29d-8a4c-41c7-b389-3e290aab7937","parentId":"02aee5af-474c-496b-b63d-79e86e540ef6","role":"user","content":"1","createdAt":"2024-09-06T06:32:12.717Z","modelId":"openai:dall-e-3","info":{"type":"userTextMessage","isSaved":True},"status":{"isLoading":False},"files":[],"parameters":{}},"selectedModel":{"id":"openai:dall-e-3","active":True,"name":"DALL·E 3","provider":"openai","maker":"openai","makerHumanName":"OpenAI","fallbackIcon":None,"providerModelId":"dall-e-3","info":{"description":"no description","fundationModel":None,"fundationModelMaker":None,"releaseDate":"2023","knowledgeCutoff":None,"context":None,"predictionTimeSecond":25,"usableAsAssistant":False,"pricing":{"inputImagePrice":0,"inputPrice":0,"outputPrice":0.04,"inputPriceUnit":"image","outputPriceUnit":"image"},"inputType":["text"],"outputType":["image"],"type":["image"],"keywords":[],"website":None,"fallbackModel":None},"parameters":[{"name":"prompt","description":"The prompt to use for the request","parameterType":"directInput","defaultValue":""},{"name":"file","description":"Files to use as complimentary information for the prompt.","parameterType":"directInput","defaultValue":[],"fileOptions":[{"fileExtension":[".c",".cpp",".cs",".java",".py",".rb",".js",".ts",".tsx",".jsx",".php",".go",".swift",".kt",".scala",".rs",".lua",".pl",".sh",".bash",".ps1",".html",".css",".scss",".less",".json",".xml",".yaml",".yml",".md",".markdown",".txt",".rtf",".tex",".log",".xlsx",".pdf",".csv",".tsv",".ini",".cfg",".conf",".toml",".sql",".r",".m",".f",".f90",".vb",".bas",".ps",".asm"],"fileExtensionName":"text","maxFileSize":5,"minFileCount":0,"maxFileCount":10}]}]},"sessionId":"c1121f53-e40c-4b9e-9321-d54bfe7af39e","userMessageId":"1753c29d-8a4c-41c7-b389-3e290aab7937","assistantMessageId":"91d4b7d7-065f-400c-8b72-634709e958b7","requestId":"07a4c955-ccef-4c97-ac92-ba5a6d5cc17d"}
# {
#     "files": [
#         {
#             "id": "a8009269-7ac8-409e-b711-b21081ca42aa",
#             "createdAt": "2024-09-06T06:31:49.758Z",
#             "name": "43fa234cc8244815979580895525f451_file.jpg",
#             "extension": "jpg",
#             "mediaType": "image",
#             "type": "jpg",
#             "uploadThingType": "url",
#             "size": 464801,
#             "url": "https://utfs.io/f/2878c70b-7fa9-4996-a3f1-d2cc9a200907-djxsni.jpg",
#         }
#     ]
# }

CHECK_PAYLOAD=[{"id":"9e8e7911-4924-47f7-9a34-c9e846c658a1","createdAt":"$D2024-09-12T09:23:24.194Z","parentId":None,"sessionId":"45be1b8a-ea37-43f5-ab80-7d53e2e7edbd","userId":"7b4943cf-78da-45bc-a38f-427a35aa3b89","role":"user","content":"1","modelId":"openai:gpt-4o-mini","info":{"type":"userTextMessage"},"status":{"isLoading":False},"files":[]}]
if __name__ == '__main__':
    import json_repair

    # print("net".startswith(("perplexity", "net")))

    print(list(GPT_4O[0]['selectedModel']))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : images
# @Time         : 2024/11/13 15:44
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://fal.ai/models/fal-ai/flux-pro/v1.1-ultra/api

from meutils.pipe import *
from meutils.str_utils import parse_url, validate_url
from meutils.io.files_utils import to_url, to_url_fal

from meutils.schemas.image_types import ImageRequest, FluxImageRequest, SDImageRequest, ImagesResponse
from meutils.schemas.fal_types import FEISHU_URL
from meutils.config_utils.lark_utils import get_next_token_for_polling, get_series

from meutils.apis.translator import deeplx

from fal_client.client import AsyncClient, SyncClient, Status, FalClientError

recraft_styles = "any, realistic_image, digital_illustration, vector_illustration, realistic_image/b_and_w, realistic_image/hard_flash, realistic_image/hdr, realistic_image/natural_light, realistic_image/studio_portrait, realistic_image/enterprise, realistic_image/motion_blur, realistic_image/evening_light, realistic_image/faded_nostalgia, realistic_image/forest_life, realistic_image/mystic_naturalism, realistic_image/natural_tones, realistic_image/organic_calm, realistic_image/real_life_glow, realistic_image/retro_realism, realistic_image/retro_snapshot, realistic_image/urban_drama, realistic_image/village_realism, realistic_image/warm_folk, digital_illustration/pixel_art, digital_illustration/hand_drawn, digital_illustration/grain, digital_illustration/infantile_sketch, digital_illustration/2d_art_poster, digital_illustration/handmade_3d, digital_illustration/hand_drawn_outline, digital_illustration/engraving_color, digital_illustration/2d_art_poster_2, digital_illustration/antiquarian, digital_illustration/bold_fantasy, digital_illustration/child_book, digital_illustration/child_books, digital_illustration/cover, digital_illustration/crosshatch, digital_illustration/digital_engraving, digital_illustration/expressionism, digital_illustration/freehand_details, digital_illustration/grain_20, digital_illustration/graphic_intensity, digital_illustration/hard_comics, digital_illustration/long_shadow, digital_illustration/modern_folk, digital_illustration/multicolor, digital_illustration/neon_calm, digital_illustration/noir, digital_illustration/nostalgic_pastel, digital_illustration/outline_details, digital_illustration/pastel_gradient, digital_illustration/pastel_sketch, digital_illustration/pop_art, digital_illustration/pop_renaissance, digital_illustration/street_art, digital_illustration/tablet_sketch, digital_illustration/urban_glow, digital_illustration/urban_sketching, digital_illustration/vanilla_dreams, digital_illustration/young_adult_book, digital_illustration/young_adult_book_2, vector_illustration/bold_stroke, vector_illustration/chemistry, vector_illustration/colored_stencil, vector_illustration/contour_pop_art, vector_illustration/cosmics, vector_illustration/cutout, vector_illustration/depressive, vector_illustration/editorial, vector_illustration/emotional_flat, vector_illustration/infographical, vector_illustration/marker_outline, vector_illustration/mosaic, vector_illustration/naivector, vector_illustration/roundish_flat, vector_illustration/segmented_colors, vector_illustration/sharp_contrast, vector_illustration/thin, vector_illustration/vector_photo, vector_illustration/vivid_shapes, vector_illustration/engraving, vector_illustration/line_art, vector_illustration/line_circuit, vector_illustration/linocut"

ideogram_styles = "auto, general, realistic, design, render_3D, anime"

# https://fal.ai/models/fal-ai/any-llm/playground
llms = "anthropic/claude-3.5-sonnet, anthropic/claude-3-5-haiku, anthropic/claude-3-haiku, google/gemini-pro-1.5, google/gemini-flash-1.5, google/gemini-flash-1.5-8b, meta-llama/llama-3.2-1b-instruct, meta-llama/llama-3.2-3b-instruct, meta-llama/llama-3.1-8b-instruct, meta-llama/llama-3.1-70b-instruct, openai/gpt-4o-mini, openai/gpt-4o, deepseek/deepseek-r1"


@alru_cache(ttl=300)
async def check(token, threshold: float = 0):
    try:
        # data = await AsyncClient(key=token).upload(b'', '', '')

        data = await AsyncClient(key=token).run(
            "fal-ai/any-llm",
            arguments={
                "model": "meta-llama/llama-3.2-1b-instruct",
                "prompt": "1+1=",
                "max_tokens": 1,
            },
        )
        logger.debug(data)
        return True
    except Exception as exc:
        logger.error(exc)
        return False


async def generate(request: ImageRequest, api_key: Optional[str] = None):
    """https://fal.ai/models/fal-ai/flux-pro/v1.1-ultra/api#api-call-submit-request
    """
    logger.debug(request)

    s = time.time()
    token = api_key or await get_next_token_for_polling(feishu_url=FEISHU_URL, from_redis=True, check_token=check)

    arguments = request.model_dump(exclude_none=True)
    width, height = (request.size or "1024x1024").split("x")
    if request.model.startswith("fal-ai/recraft"):  # https://fal.ai/models/fal-ai/recraft-v3/api#queue-submit
        arguments = {
            **arguments,
            "image_size": {
                "width": width,
                "height": height
            },
            "style": request.style if str(request.style) in recraft_styles else "realistic_image",

            "output_format": "png",
        }
    elif request.model.startswith("fal-ai/flux/dev/image-to-image"):
        urls = parse_url(request.prompt)
        image_url = urls[-1]
        prompt = request.prompt.replace(image_url, "")

        arguments = {
            **arguments,
            "image_size": {
                "width": width,
                "height": height
            },
            "num_images": request.n or 1,

            "enable_safety_checker": False,
            "safety_tolerance": "6",
            "output_format": "png",
            "seed": request.seed,

            # image2image
            "image_url": image_url,
            "prompt": prompt,
            "guidance_scale": request.guidance or 3.5,

            **request.controls
        }
    elif "kontext" in request.model:  # https://fal.ai/models/fal-ai/flux-pro/kontext/max

        if (image_urls := request.image_urls) or (image_urls := parse_url(request.prompt)):
            if not validate_url(image_urls):
                # raise Exception(f"Invalid image url: {image_urls}")
                from fastapi import HTTPException, status
                raise HTTPException(
                    status_code=status.HTTP_451_UNAVAILABLE_FOR_LEGAL_REASONS,
                    detail=f"Invalid image url: {image_urls}",
                )

            for image_url in image_urls:
                request.prompt = request.prompt.replace(image_url, "")

            if len(image_urls) == 1:
                arguments['image_url'] = image_urls[0]
            else:
                request.model += "/multi"
                arguments['image_urls'] = image_urls
        else:
            request.model += "/text-to-image"

        # Path /kontext/text-to-image/text-to-image not found

        # fal-ai/flux-pro/kontext/max/text-to-image

        # fal-ai/flux-pro/kontext/max
        # fal-ai/flux-pro/kontext/max/multi

        # fal-ai/flux-pro/kontext/text-to-image

        # fal-ai/flux-pro/kontext
        # fal-ai/flux-pro/kontext/multi

        """
        {
          "prompt": "Put a donut next to the flour.",
          "guidance_scale": 3.5,
          "num_images": 1,
          "safety_tolerance": "2",
          "output_format": "jpeg",
          "image_url": "https://v3.fal.media/files/rabbit/rmgBxhwGYb2d3pl3x9sKf_output.png"
        }
        """

        arguments = {
            **arguments,

            "num_images": request.n or 1,

            "enable_safety_checker": False,
            "safety_tolerance": "5",
            "output_format": "png",
            "seed": request.seed,

            "guidance_scale": request.guidance or 3.5,

            "model": request.model,
            "prompt": await deeplx.llm_translate(request.prompt),

        }

        if request.aspect_ratio:
            arguments.pop("aspect_ratio", None)

            aspect_ratios = {'21:9', '16:9', '4:3', '3:2', '1:1', '2:3', '3:4', '9:16', '9:21'}

            if aspect_ratio := request.prompt2aspect_ratio(aspect_ratios):  # 提示词优先级最高
                arguments["aspect_ratio"] = aspect_ratio

            elif request.aspect_ratio in aspect_ratios:
                arguments["aspect_ratio"] = request.aspect_ratio

        logger.debug(bjson(arguments))

    elif request.model.startswith("fal-ai/flux"):  # https://fal.ai/models/fal-ai/flux-pro/v1.1-ultra/api

        arguments = {
            **arguments,
            "num_images": request.n or 1,

            "enable_safety_checker": False,
            "safety_tolerance": "6",
            "output_format": "png",
            "seed": request.seed,

            "aspect_ratio": request.aspect_ratio or "1:1"  # 21:9, 16:9, 4:3, 3:2, 1:1, 2:3, 3:4, 9:16, 9:21

        }

    elif request.model.startswith("fal-ai/ideogram"):
        arguments = {
            **arguments,
            "style": request.style.upper() if str(request.style).lower() in ideogram_styles else "AUTO",

            # 10:16, 16:10, 9:16, 16:9, 4:3, 3:4, 1:1, 1:3, 3:1, 3:2, 2:3
            "aspect_ratio": request.aspect_ratio or "1:1",

            "seed": request.seed,

        }

    elif request.model.startswith("fal-ai/imagen4"):
        arguments = {
            **arguments,
            # 10:16, 16:10, 9:16, 16:9, 4:3, 3:4, 1:1, 1:3, 3:1, 3:2, 2:3
            "aspect_ratio": request.aspect_ratio or "1:1",

            "seed": request.seed,

        }

    elif request.model.startswith(("fal-ai/nano-banana", "fal-ai/gemini-25-flash-image")):
        arguments = {
            "prompt": request.prompt,
            "image_urls": request.image_urls,
            "num_images": request.n or 1,
            "output_format": "png"
        }
        if request.image_urls:
            request.model = f"""{request.model.removesuffix("/edit")}/edit"""

        logger.debug(bjson(arguments))

    try:

        data = await AsyncClient(key=token).run(
            application=request.model,
            arguments=arguments,
        )
        logger.debug(data)
        return ImagesResponse(data=data.get("images"), timings={"inference": time.time() - s})

    except Exception as exc:  #
        logger.error(exc)
        # from fastapi import HTTPException, status
        # raise HTTPException(
        #     status_code=status.HTTP_451_UNAVAILABLE_FOR_LEGAL_REASONS,
        #     detail=f"Prompt is sensitive.\n\n{request.prompt}\n\n{exc}",
        # )

        raise exc


if __name__ == '__main__':
    mapper = \
        {
            "flux-pro-1.1": "fal-ai/flux-pro/v1.1",
            "flux-pro-1.1-ultra": "fal-ai/flux-pro/v1.1-ultra",
            "ideogram-ai/ideogram-v2-turbo": "fal-ai/ideogram/v2/turbo",
            "ideogram-ai/ideogram-v2": "fal-ai/ideogram/v2",
            "recraftv3": "fal-ai/recraft-v3",

            "imagen4": "fal-ai/imagen4/preview",
            "flux-kontext-pro": "fal-ai/flux-pro/kontext",
            "flux-kontext-max": "fal-ai/flux-pro/kontext/max"

        }

    # model = "fal-ai/flux-pro/v1.1"  # 0.04
    model = "fal-ai/flux-pro/v1.1-ultra"  # 0.06
    # model = "fal-ai/flux/dev/image-to-image"  # 0.03 图生图
    # model = "fal-ai/recraft-v3"  # 0.04 变清晰
    # model = "fal-ai/ideogram/v2" # 0.08
    # model = "fal-ai/ideogram/v2/turbo"  # 0.05
    # model = "fal-ai/flux/schnell"
    model = "fal-ai/imagen4/preview"
    model = "fal-ai/flux-pro/kontext"
    # model = "fal-ai/flux-pro/kontext/max"

    model = "fal-ai/nano-banana"

    #
    prompt = "https://v3.fal.media/files/penguin/XoW0qavfF-ahg-jX4BMyL_image.webp https://v3.fal.media/files/tiger/bml6YA7DWJXOigadvxk75_image.webp Put the little duckling on top of the woman's t-shirt."
    # prompt = '把小鸭子放在女人的T恤上面。\nhttps://s3.ffire.cc/cdn/20250530/tEzZKkhp3tKbNzva6mgC2T\nhttps://s3.ffire.cc/cdn/20250530/AwHJpuJuNg5w3sVbH4PZdv'
    request = ImageRequest(prompt=prompt, model=model)

    data = {
        "model": "fal-ai/flux-pro/kontext",  # flux-kontext-pro
        # "prompt": "https://s3.ffire.cc/files/3_4.webp 老虎换个姿势",
        "prompt": "保持服装 发型，面纱细节不变，转换为真实拍摄的照片风格，一个年轻的中国顶级颜值的演员扮演这个角色。",
        "size": 'auto'
    }

    # data = {
    #     "model": "fal-ai/gemini-25-flash-image",
    #     "prompt": "make a photo of the man driving the car down the california coastline",
    #     "image": [
    #         "https://storage.googleapis.com/falserverless/example_inputs/nano-banana-edit-input.png",
    #         "https://storage.googleapis.com/falserverless/example_inputs/nano-banana-edit-input-2.png"
    #     ],
    #     "num_images": 1
    # }
    #
    #
    data = {
        # "model": "fal-ai/gemini-25-flash-image",
        "model": "fal-ai/nano-banana",
        ""
        "prompt": "裸体女孩",
        # "image": [
        #     "https://storage.googleapis.com/falserverless/example_inputs/nano-banana-edit-input.png",
        #     "https://storage.googleapis.com/falserverless/example_inputs/nano-banana-edit-input-2.png"
        # ],
        "num_images": 1
    }

    data = {
        "model": "fal-ai/ideogram/v3",
        # "rendering_speed": "BALANCED",
        "expand_prompt": True,
        "num_images": 1,
        "prompt": "The Bone Forest stretched across the horizon, its trees fashioned from the ossified remains of ancient leviathans that once swam through the sky. Shamans with antlers growing from their shoulders and eyes that revealed the true nature of any being they beheld conducted rituals to commune with the spirits that still inhabited the calcified grove. In sky writes \"Ideogram V3 in fal.ai\"",
        "image_size": "square_hd",
        "image_urls": [],
        "style": "xxx"
    }

    request = ImageRequest(**data)
    print(request)

    api_key = "5218f1bb-360b-4719-847c-d611447d96e0:c11599dd540ad9d99bd9280399d0ba3f"

    arun(generate(request, api_key))

    # request = ImageRequest(prompt='https://oss.ffire.cc/files/kling_watermark.png The woman smiled', model=model)
    #
    # request = ImageRequest(prompt="https://oss.ffire.cc/files/kling_watermark.png 让这个女人哭起来", model=model)
    #

    # request = ImageRequest(prompt='a cat', model=model)

    # token = "77fcfad2-aadc-4158-8e11-b30a5668dfad:ea5f38d864bde6707561d348436d2cea"
    # token = "5116af3f-1aa5-4c1b-b505-fd49f701dccc:7f4278d69d069cec4794fe658a09bd9d"

    # arun(generate(request))

    # arun(AsyncClient(key=token).key)

    # FEISHU_URL = "https://xchatllm.feishu.cn/sheets/QB6Psj8zrhB4JStEW0acI4iInNg?sheet=ef2e81"
    #
    # tokens = arun(get_series(FEISHU_URL))
    # tokens = ["578e143c-702d-46a1-8c02-d451bc3e155b:bb531a95f79c6026e7bc7ab676163c71"]
#     tokens = """
# 30e9fe5c-75d0-4bdd-b11d-ff17dbe4bba5:3de8535e2fa4d9347e599a845cf3e7b8
# 05c0b4f6-1443-4b28-aeff-771b65ebaebc:28df6b9b71430b05aca6b8bc7e873a4a
#     """.split()
#
#     r = []
#     for t in tokens:
#         if arun(check(t)):
#             r.append(t)
#
#     print('\n'.join(r))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : demo
# @Time         : 2024/10/25 22:17
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import json

from meutils.pipe import *
from meutils.apis.hailuoai.hasha import js_code

import execjs

# 编译 JavaScript 代码
context = execjs.compile(js_code)

# 调用 JavaScript 函数
result = context.call("exports", "1729863023000")
print(result)  # 输出: Hello, World

url = "https://hailuoai.com/v1/api/files/policy_callback?device_platform=web&app_id=3001&version_code=22201&uuid=8c059369-00bf-4777-a426-d9c9b7984ee6&device_id=243713252545986562&os_name=Mac&browser_name=chrome&device_memory=8&cpu_core_num=10&browser_language=zh-CN&browser_platform=MacIntel&screen_width=1352&screen_height=878&unix=1729865709000"

payload = json.dumps({"fileName": "2300140d-a576-408b-ba89-40541786a6e4.png",
                      "originFileName": "503af3b5-9c3b-4bdc-a6d4-256debce3dd5_00001_.png",
                      "dir": "cdn-yingshi-ai-com/prod/2024-10-25-22/user/multi_chat_file",
                      "endpoint": "oss-cn-wulanchabu.aliyuncs.com", "bucketName": "minimax-public-cdn",
                      "size": "1681865", "mimeType": "png", "fileMd5": "923e10167a2d7b36e866319dad738b1e",
                      "fileScene": 10})

s = f"""/v1{url.split('/v1', 1)[-1]}_{payload}{context.call("exports", url.rsplit('unix=', 1)[-1])}"""




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : metaso_types
# @Time         : 2024/11/11 17:26
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *

BASE_URL = "https://metaso.cn"
FEISHU_URL = "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=cyKbvv"


class MetasoRequest(BaseModel):
    model: Optional[Literal["ds-r1", "fast_thinking"]] = None

    """search-mini search search-pro
    
    model-mode
    
    """
    mode: Union[str, Literal["concise", "detail", "research", "strong-research"]] = "detail"  # concise detail research

    question: str = "Chatfire"

    """全网 文库 学术 图片 播客 视频"""
    scholarSearchDomain: str = "all"
    engineType: Optional[Literal["pdf", "scholar", "image", "podcast", "video"]] = None

    url: str = "https://metaso.cn/"
    lang: str = "zh"

    searchTopicId: Optional[str] = None
    searchTopicName: Optional[str] = None

    enableMix: str = 'true'
    newEngine: str = 'true'
    enableImage: str = 'true'

#

# question: hi
# mode: detail
# scholarSearchDomain: all
# model: ds-r1
# url: https://metaso.cn/
# lang: zh
# enableMix: true
# newEngine: true
# enableImage: true
# metaso-pc: pc
class MetasoResponse(BaseModel):  # sse

    type: Optional[str] = None  # query set-reference heartbeat append-text
    content: str = ""

    data: Optional[dict] = None
    references: list = []

    # 原生内容
    chunk: str

    def __init__(self, /, **data: Any):
        super().__init__(**data)

        chunk = self.chunk.lstrip("data:")
        self.data = json.loads(chunk)

        self.type = self.data.get("type")
        self.content = self.data.get("text", "")

        # {'realQuestion': '你是谁', 'data': [], 'label': '', 'id': '8544588308750417920', 'type': 'query'}
        if self.type == "query":
            self.data.pop("id", None)
            self.data.pop("debugId", None)
            self.content = f"""> 🚀AISearch\n```json\n{self.data}\n```\n\n"""

        if self.type in {"set-reference", "update-reference"}:
            self.references = self.data.get("list", [])


if __name__ == '__main__':
    chunk = """data:{"type":"heartbeat"}"""

    print(MetasoResponse(chunk=chunk))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : generations
# @Time         : 2025/6/11 17:06
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 统一收口 todo 硅基
import os

from meutils.pipe import *
from meutils.llm.clients import AsyncClient
from meutils.llm.openai_utils import to_openai_params
from meutils.io.files_utils import to_png, to_url_fal, to_url
from meutils.notice.feishu import send_message_for_images

from meutils.schemas.image_types import ImageRequest, RecraftImageRequest, ImagesResponse

from meutils.apis.fal.images import generate as fal_generate

from meutils.apis.gitee.image_to_3d import generate as image_to_3d_generate
from meutils.apis.gitee.openai_images import generate as gitee_images_generate
from meutils.apis.volcengine_apis.images import generate as volc_generate
from meutils.apis.images.recraft import generate as recraft_generate
from meutils.apis.jimeng.images import generate as jimeng_generate
# from meutils.apis.google.images import generate as google_generate

from meutils.apis.qwen.chat import Completions as QwenCompletions
from meutils.apis.google.chat import Completions as GoogleCompletions
from meutils.apis.google.images import openai_generate
from meutils.apis.images.runware import generate as runware_generate


async def generate(
        request: ImageRequest,
        api_key: Optional[str] = None,
        base_url: Optional[str] = None,
):
    if len(str(request)) < 1024:
        logger.debug(request)

    if request.model.startswith("fal-ai"):  # 国外fal
        request.image = await to_url_fal(request.image, content_type="image/png")
        response = await fal_generate(request, api_key)

        # https://fal.media/files/panda/FlN5Gk0KnHe4AXU6Jeyvo_c5795dc52214423cb6465dae8eeaa1f0.png
        # https://s3ai.cn/fal/files/panda/FlN5Gk0KnHe4AXU6Jeyvo_c5795dc52214423cb6465dae8eeaa1f0.png

        #

        # """https://fal.media/files/b/rabbit/-Cvx3pnaB0p6p_RLXlfuo.png
        # """
        # 转存
        # if response.data and request.response_format == "oss_url":
        #     urls = [dict(image_data).get("url") for image_data in response.data]
        #     urls = await to_url(urls, filename=f"{shortuuid.random()}.png")
        #
        #     response.data = [{"url": url} for url in urls]
        #
        #     return response

        if response.data and request.response_format == "oss_url":
            urls = [
                f"""https://s3ai.cn/fal/files/panda/{Path(dict(image_data).get("url", "")).name}"""

                for image_data in response.data
            ]

            response.data = [{"url": url} for url in urls] + response.data

        send_message_for_images(response.data, title=__file__)

        return response

    if request.model.startswith(("recraft",)):
        request = RecraftImageRequest(**request.model_dump(exclude_none=True))
        return await recraft_generate(request)

    if request.model.startswith(
            ("jimeng", "seed", "seededit_v3.0", "byteedit_v2.0", "i2i_portrait_photo")):  # seededit seedream
        return await volc_generate(request, api_key)

    if request.model.startswith(("jimeng")):  # 即梦 逆向
        return await jimeng_generate(request)

    if request.model in {"Hunyuan3D-2", "Hi3DGen", "Step1X-3D"}:
        return await image_to_3d_generate(request, api_key)

    if request.model in {"Qwen-Image", "FLUX_1-Krea-dev"} and request.model.endswith(("lora",)):
        return await gitee_images_generate(request, api_key)

    if request.model.startswith("qwen-image"):
        return await QwenCompletions(api_key=api_key).generate(request)

    if request.model.startswith(("google/gemini", "gemini")):  # openrouter
        if "ppi" in base_url:  # 派欧 todo bug
            request.images = request.image
            pass

        elif api_key.endswith("-openai"):
            api_key = api_key.removesuffix("-openai")
            return await openai_generate(request, base_url=base_url, api_key=api_key)
        else:
            return await GoogleCompletions(base_url=base_url, api_key=api_key).generate(request)  # 原生接口

    if request.model.startswith("runware") or all(i in request.model for i in {":", "@"}):
        return await runware_generate(request, api_key)

    # 其他
    data = {
        **request.model_dump(exclude_none=True, exclude={"extra_fields", "aspect_ratio"}),
        **(request.extra_fields or {})
    }
    request = ImageRequest(**data)
    if request.model.startswith("doubao"):
        base_url = base_url or os.getenv("VOLC_BASE_URL")
        api_key = api_key.split()[0]  # 一个号并发足够

        request.stream = False
        request.watermark = False
        if request.model.startswith("doubao-seedream-4"):
            if (
                    request.image
                    and not any(
                i in str(request.image).lower() for i in {".png", ".jpg", ".jpeg", "image/png", "image/jpeg"}
            )
            ):
                logger.debug(f"{request.model}: image 不是 png 或 jpeg 格式，转换为 png 格式")

                request.image = await to_png(request.image, response_format='b64')

            if request.n > 1:
                request.sequential_image_generation = "auto"
                request.sequential_image_generation_options = {
                    "max_images": request.n
                }
        elif request.image and isinstance(request.image, list):
            request.image = request.image[0]

        if "ppi" in base_url:  # 派欧 https://ppio.com/docs/models/reference-seedream4.0 images => image
            request.images = request.image

    data = to_openai_params(request)

    if len(str(data)) < 1024:
        logger.debug(bjson(data))

    client = AsyncClient(api_key=api_key, base_url=base_url)
    return await client.images.generate(**data)


# "flux.1-krea-dev"

if __name__ == '__main__':
    # arun(generate(ImageRequest(model="flux", prompt="笑起来")))
    # arun(generate(ImageRequest(model="FLUX_1-Krea-dev", prompt="笑起来")))

    token = f"""{os.getenv("VOLC_ACCESSKEY")}|{os.getenv("VOLC_SECRETKEY")}"""
    # arun(generate(ImageRequest(model="seed", prompt="笑起来"), api_key=token))

    request = ImageRequest(model="doubao-seedream-4-0-250828", prompt="a dog", size="1K")

    request = ImageRequest(
        model="doubao-seedream-4-0-250828",
        prompt="将小鸭子放在t恤上,生成1:2比例图",
        size="4k",
        image=[
            "https://v3.fal.media/files/penguin/XoW0qavfF-ahg-jX4BMyL_image.webp",
            "https://v3.fal.media/files/tiger/bml6YA7DWJXOigadvxk75_image.webp"
        ]
    )

    # todo: tokens 4096 1张

    # 组图
    # request = ImageRequest(
    #     model="doubao-seedream-4-0-250828",
    #     prompt="参考这个LOGO，做一套户外运动品牌视觉设计，品牌名称为GREEN，包括包装袋、帽子、纸盒、手环、挂绳等。绿色视觉主色调，趣味、简约现代风格",
    #     image="https://ark-project.tos-cn-beijing.volces.com/doc_image/seedream4_imageToimages.png",
    #     n=3
    # )

    # arun(generate(request, api_key=os.getenv("FFIRE_API_KEY"), base_url=os.getenv("FFIRE_BASE_URL")))  # +"-29494"

    # print(not any(i in str(request.image) for i in {".png", ".jpeg", "image/png", "image/jpeg"}))

    api_key = "sk_fRr6ieXTMfym7Q6cnbj0YBlB1QsE74G8ygqIE2AyGz0"
    base_url = "http://all.chatfire.cn/ppinfra/v1"

    arun(generate(request, api_key=api_key, base_url=base_url))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : images
# @Time         : 2025/10/9 11:55
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import httpx

from meutils.pipe import *

from meutils.schemas.image_types import ImageRequest


async def generate(request: ImageRequest, api_key: str, base_url: Optional[str] = None):
    if request.model.startswith("gemini"):
        # model = "gemini-2.5-flash-image-preview"
        if request.image:
            request.model += "-image-edit"
        else:
            request.model += "-text-to-image"

    # gemini-2.5-flash-image-preview-text-to-image
    base_url = f"https://api.ppinfra.com/v3/{request.model}"

    async with httpx.AsyncClient(timeout=120) as client:
        response = await client.post(
            base_url,
            headers={
                "Authorization": f"Bearer {api_key}",
                "Content-Type": "application/json",
            },
            json={
                "prompt": request.prompt,
                # "image_urls": request.image_urls,
            },
        )
        response.raise_for_status()
        return response.json()


if __name__ == '__main__':
    request = ImageRequest(
        model="gemini-2.5-flash-image-preview",
        prompt="a cat",
    )

    arun(generate(request, api_key="sk_fRr6ieXTMfym7Q6cnbj0YBlB1QsE74G8ygqIE2AyGz0"))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : rag
# @Time         : 2024/11/21 16:58
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
import os
from pathlib import Path
from openai import OpenAI

client = OpenAI(
    api_key=os.getenv("DASHSCOPE_API_KEY"),
    base_url="https://dashscope.aliyuncs.com/compatible-mode/v1",
)

file_object = client.files.create(file=Path("百炼系列手机产品介绍.docx"), purpose="file-extract")
print(file_object.id) # file-fe-fUEnuSjIEbuMnKWpWSPKWGVe

# print(client.files.retrieve_content("file-fe-fUEnuSjIEbuMnKWpWSPKWGVe"))
#
# completion = client.chat.completions.create(
#     model="qwen-turbo-2024-11-01",
#     messages=[
#         {'role': 'system', 'content': 'You are a helpful assistant.'},
#         {'role': 'system', 'content': f'fileid://{file_object.id}'},
#         {'role': 'user', 'content': '这篇文章讲了什么？'}
#     ],
#     stream=True,
#     stream_options={"include_usage": True}
# )
#
# for chunk in completion:
#     print(chunk.model_dump())


print(client.files.content(file_id=file_object.id).text)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : ann_inmemory
# @Time         : 2023/5/15 17:50
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import numpy as np
import pandas as pd

from meutils.pipe import *
from meutils.np_utils import cosine


def ann_find(v, from_array, topk=5, threshold=0.5):
    """适合万级向量匹配"""
    if np.array(v).ndim == 1:
        v = np.array(v).reshape(1, -1)

    dist = - cosine(v, from_array)
    idxs = np.argsort(dist)[:, :topk]
    scores = - np.take_along_axis(dist, idxs, -1)
    _ = (
        pd.DataFrame(
            {
                'id': idxs[0],
                'score': scores[0],
                'embedding': from_array[idxs[0]].tolist()
            }
        )
        .query(f"score > {threshold}")
    )

    return _


if __name__ == '__main__':
    x = np.random.rand(10, 128)
    print(ann_find(x[0], x))
    print(ann_find(x[:1], x))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : doc_prompt
# @Time         : 2023/9/1 19:11
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

import cv2
from paddlenlp.taskflow.document_intelligence import DocPromptTask


def _preprocess(self, inputs):
    """
    Transform the raw text to the model inputs, two steps involved:
       1) Transform the raw text to token ids.
       2) Generate the other model inputs from the raw text and token ids.
    """
    preprocess_results = self._check_input_text(inputs)
    ocr_result = []
    for example in preprocess_results:
        if "word_boxes" in example.keys():
            ocr_result = example["word_boxes"]
            example["ocr_type"] = "word_boxes"
        else:
            ocr_result = self._ocr.ocr(example["doc"], cls=True)
            example["ocr_type"] = "ppocr"
            # Compatible with paddleocr>=2.6.0.2
            ocr_result = ocr_result[0] if len(ocr_result) == 1 else ocr_result
        example["ocr_result"] = ocr_result
    self.ocr_result = ocr_result
    self.ocr_result_map = {r[1][0]: r[0] for r in ocr_result}
    return preprocess_results


DocPromptTask._preprocess = _preprocess
DocPromptTask._check_input_text = _check_input_text


def result2box(result, docprompt):
    ocr_result_map = docprompt.task_instance.ocr_result_map

    result = result.copy()
    results = result.get('result', [])
    for r in results:
        word = r.get('value', '')
        for w, b in ocr_result_map.items():
            if word in w:
                r['box'] = b
    #                 continue
    return result


def image2path(image):
    filename = f"{uuid.uuid1()}.png"
    cv2.imwrite(filename, image)
    return filename


from paddlenlp import Taskflow

# device_id=-1 本地测试用cpu ，生产可以切换成gpu，删除参数即可，默认gpu
docprompt = Taskflow("document_intelligence")


if __name__ == '__main__':
    # p = "/Users/betterme/PycharmProjects/AI/MeUtils/meutils/ai_cv/invoice.jpg"
    # docs = [
    #     {"doc": p, "prompt": ["发票号码是多少?", "校验码是多少?"]},
    # ]
    #
    # res = docprompt(docs)
    #
    # res = list(map(lambda r: result2box(r, docprompt=docprompt), res))
    #
    # print(res)

    p = "/Users/betterme/PycharmProjects/AI/MeUtils/meutils/ai_cv/invoice.jpg"
    import cv2
    image = cv2.imread(p)

    docs = [
        {"doc": image2path(image), "prompt": ["发票号码是多少?", "校验码是多少?"]},
    ]

    res = docprompt(docs)

    res = list(map(lambda r: result2box(r, docprompt=docprompt), res))

    print(res)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : kolors
# @Time         : 2024/7/25 08:42
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://huggingface.co/spaces/gokaygokay/KolorsPlusPlus
# https://huggingface.co/spaces/Kwai-Kolors/Kolors-Virtual-Try-On

from meutils.pipe import *
from meutils.schemas.openai_types import ImageRequest, ImagesResponse
from meutils.async_utils import sync_to_async
from meutils.decorators.retry import retrying

from gradio_client import Client as _Client, handle_file
from meutils.io.image import image_to_base64

from gradio_client import Client

# client = Client("https://cunyue-person-modnet.demo.swanhub.co/")
#
#
#
# url = "https://cunyue-person-modnet.demo.swanhub.co/file=/app/image/image04.jpg"
# result = client.predict(
#     data=[image_to_base64(url)],
#     # str (filepath or URL to image) in 'Image' Image component
#     api_name="/predict"
# )
# print(result)

from gradio_client import Client

client = Client("https://s5k.cn/api/v1/studio/binzhouchn/try-on-cloth/gradio/", hf_token="a22224b9-b276-45b5-b8d1-10f3c81c8eec")
result = client.predict(
		name="Hello!!",
		api_name="/predict"
)
print(result)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : sdk
# @Time         : 2025/1/17 10:54
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *


client_id = "6e4ca6247dea4186a09fc64362f72e51"
client_secret = "114d33c9-079b-2684-f93a-e6a1031260e1"

# BAIDU_AI=38243958:ayIjRRUv3MNNpss9r4YkCIKF:NqcBofQtvdAWKtiGAMjIkASWNxCKW8Fv # 25308860:GmAPqlyBDNLaoAqO2mrFhjS2:vXIoxFUdx2jiuRnGLvZSDMEczEaZsc1K

# DZjHpNLUauqYNFgpNujNhH5p 0p3XGiy0Y7troeGukFiEnlRMsq5smw9W
def main(client_id, client_secret):
    url = f"https://aip.baidubce.com/oauth/2.0/token?grant_type=client_credentials&client_id={client_id}&client_secret={client_secret}"

    payload = ""
    headers = {
        'Content-Type': 'application/json',
        'Accept': 'application/json'
    }

    response = requests.request("POST", url, headers=headers, data=payload)

    print(response.text)


from baidubce.bce_client_configuration import BceClientConfiguration
from baidubce.auth.bce_credentials import BceCredentials
from baidubce.services.bos import storage_class, bos_client

# 设置BosClient的Host，Access Key ID和Secret Access Key
bos_host = "hkg.bcebos.com"
access_key_id = "ALTAKRFJAQgNsEt2FTdrr5avOh"
secret_access_key = "b45cee038ca14852a8be3697595dec28"
config = BceClientConfiguration(credentials=BceCredentials(access_key_id, secret_access_key), endpoint=bos_host)

client = bos_client.BosClient(config)

if __name__ == '__main__':
    # client_id="DZjHpNLUauqYNFgpNujNhH5p"
    # client_secret = "0p3XGiy0Y7troeGukFiEnlRMsq5smw9W"
    # main(client_id, client_secret)
    user_metadata = {"name": "my-data"}


    _ = client.put_object_from_string("chatfire", "object_key", data="data")




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : text_split_with_overlap
# @Time         : 2023/4/25 15:33
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from meutils.ai_nlp.SplitSentence import split_sentence


def merge_short_sentences(sentences, chunk_size):
    short_sentences = []
    for i, sentence in enumerate(sentences):  # 句子太长 需要再切
        short_sentences.append(sentence)
        if len(''.join(short_sentences)) > chunk_size:
            return sentences[:i], sentences[i:]
    return sentences, []


def textsplitter_with_overlap(sentences, chunk_size=512, chunk_overlap_rate=0.2):
    """

    :param sentences: sentences = split_sentence(text)
    :param chunk_size:
    :param chunk_overlap_rate:
    :return:
    """
    chunk_overlap = int(chunk_size * chunk_overlap_rate)

    result = []
    while sentences:
        merge_sentences, sentences = merge_short_sentences(sentences, chunk_size)
        # result.append(merge_sentences)
        result.append(''.join(merge_sentences).split() | xjoin)

        if not sentences:
            break

        overlap_sentences = merge_short_sentences(merge_sentences[::-1], chunk_overlap)[0][::-1]

        if len(''.join(overlap_sentences)) + len(sentences[0]) > chunk_size:  # 丢弃重叠部分
            continue

        sentences = overlap_sentences + sentences  # 新句子集合
    return result



if __name__ == '__main__':
    text = '央视新闻消息，近日，特朗普老友皮尔斯·摩根喊话特朗普：“美国人的生命比你的选举更重要。如果你继续以自己为中心，继续玩弄愚蠢的政治……如果你意识不到自己>的错误，你就做不对”。目前，特朗普已“取关”了这位老友。'
    sentences = split_sentence(text, criterion='coarse')
    print(sentences)
    print(textsplitter_with_overlap(sentences))

    from langchain.text_splitter import CharacterTextSplitter

    print(CharacterTextSplitter().split_text(text))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : runware
# @Time         : 2025/10/9 16:34
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from meutils.db.redis_db import redis_aclient
from meutils.llm.clients import AsyncClient
from meutils.notice.feishu import send_message_for_images
from meutils.io.files_utils import to_url, to_url_fal
from meutils.schemas.image_types import ImageRequest, RecraftImageRequest, ImagesResponse
from openai import BadRequestError

from meutils.apis.translator import deeplx


async def generate(request: ImageRequest, api_key: str, base_url: Optional[str] = None):
    payload = {
        **request.model_dump(exclude_none=True, exclude={"extra_fields", "aspect_ratio"}),
        **(request.extra_fields or {})
    }
    payload = [
        {
            "taskUUID": str(uuid.uuid4()),
            "taskType": "imageInference",
            "model": request.model,
            "positivePrompt": await deeplx.llm_translate(request.prompt),

            "numberResults": request.n,
            # "width": 1024,
            # "height": 1024,  # aspect_ratio todo 如何映射到 width/height
            "aspectRatio": request.aspect_ratio,

            "includeCost": True,
            "outputFormat": "PNG",
            "outputType": [
                "URL"
            ],

            **payload
        }
    ]

    if image_urls := request.image_urls:
        if not image_urls[0].startswith("http"):  # 转换为url
            image_urls = await to_url_fal(image_urls, content_type="image/png")

        payload[0]["referenceImages"] = image_urls

    if aspect_ratio_mapping := await redis_aclient.get(f"runware:{request.model}"):
        aspect_ratio_mapping = aspect_ratio_mapping.decode()
        logger.debug(aspect_ratio_mapping)
        aspect_ratio_mapping = json.loads(aspect_ratio_mapping)

        size = aspect_ratio_mapping.get(request.aspect_ratio)
        if size and isinstance(size, list):
            size = size[-1]  # 取最高清

        payload[0]["width"], payload[0]["height"] = map(int, size.split("x"))

    logger.debug(bjson(payload))
    try:
        client = AsyncClient(base_url="https://api.runware.ai/v1", api_key=api_key, timeout=300)
        response = await client.post(
            "/",
            body=payload,
            cast_to=object
        )

        if data := response.get("data"):
            for d in data:
                d["url"] = d["imageURL"]

            payload[0]["data"] = data
            send_message_for_images(payload[0], title=__file__)  # 存图片

            return ImagesResponse(data=data)
        elif errors := response.get("errors"):
            raise Exception(errors[0].get("message"))

    except BadRequestError as e:
        _ = e.response.json().get("errors")[0].get("allowedValues", "")  # 缓存redis
        logger.error(bjson(_))

        await redis_aclient.set(f"runware:{request.model}", json.dumps(_), ex=30 * 24 * 3600)

        raise e


if __name__ == '__main__':
    model = "google:2@1"
    model = "google:2@3"
    # model = "bfl:3@1"
    # prompt = "一个裸体少女"
    prompt = "a cat"
    request = ImageRequest(model=model, prompt=prompt, aspect_ratio="1:1")

    # request = ImageRequest(
    #     model=model,
    #     prompt="将鸭子放在女人的t恤上",
    #     aspect_ratio="1:1",
    #     image=[
    #         "https://v3.fal.media/files/penguin/XoW0qavfF-ahg-jX4BMyL_image.webp",
    #         "https://v3.fal.media/files/tiger/bml6YA7DWJXOigadvxk75_image.webp"
    #     ]
    # )

    logger.debug(request)
    arun(generate(request, api_key="Fk3Clsgcwc3faIvbsjDajGFATJLfaWpE"))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : rag
# @Time         : 2024/11/21 16:58
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
import os
from pathlib import Path
from openai import OpenAI

client = OpenAI(
    api_key=os.getenv("DASHSCOPE_API_KEY"),
    base_url="https://dashscope.aliyuncs.com/compatible-mode/v1",
)

file_object = client.files.create(file=Path("百炼系列手机产品介绍.docx"), purpose="file-extract")
print(file_object.id) # file-fe-fUEnuSjIEbuMnKWpWSPKWGVe

# print(client.files.retrieve_content("file-fe-fUEnuSjIEbuMnKWpWSPKWGVe"))

completion = client.chat.completions.create(
    model="qwen-long",
    messages=[
        {'role': 'system', 'content': 'You are a helpful assistant.'},
        {'role': 'system', 'content': f'fileid://{file_object.id}'},
        {'role': 'user', 'content': '这篇文章讲了什么？'}
    ],
    stream=True,
    stream_options={"include_usage": True}
)

for chunk in completion:
    print(chunk.model_dump())


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : check_api_key
# @Time         : 2024/6/17 14:56
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from meutils.async_utils import async_to_sync

from meutils.config_utils.lark_utils import get_spreadsheet_values
from meutils.notice.feishu import send_message as _send_message

from meutils.db.redis_db import redis_client
from meutils.llm.check_api import check_api_key_or_token

send_message = partial(
    _send_message,
    url="https://open.feishu.cn/open-apis/bot/v2/hook/e0db85db-0daf-4250-9131-a98d19b909a9",
)


@cli.command()
def check_and_update_api_keys(check_url: Optional[str], feishu_url, ttl: Optional[int] = None):
    """
    python check_api.py https://api.deepseek.com/user/balance https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=X0ZN3H


    """

    api_keys = get_spreadsheet_values(feishu_url=feishu_url, to_dataframe=True)[0].tolist()

    if feishu_url in redis_client:
        api_keys += redis_client.lrange(feishu_url, 0, -1) | xmap(lambda x: x.decode())

    # 有效 api_keys
    api_keys = async_to_sync(check_api_key_or_token)(set(api_keys), check_url)

    if feishu_url in redis_client:
        lastest_api_keys = redis_client.lrange(feishu_url, 0, -1) | xmap_(lambda x: x.decode())

        # 新增的
        to_update_api_keys = list(set(api_keys) - set(lastest_api_keys))
        to_update_api_keys += lastest_api_keys | xfilter_(lambda api_key: api_key in api_keys)  # 存量有效的&按当前顺序

    else:
        to_update_api_keys = api_keys

    num = 0
    if to_update_api_keys:  # 更新 redis
        to_update_api_keys = to_update_api_keys | xUnique

        redis_client.delete(feishu_url)
        num = redis_client.rpush(feishu_url, *to_update_api_keys)

        if ttl: redis_client.expire(feishu_url, ttl)  # Redis 的 RPUSH 命令本身不支持直接设置过期时间

    api_keys_str = (to_update_api_keys and to_update_api_keys[:20]) | xjoin('\n')
    send_message(f"有效轮询 {num} 个\n\n{feishu_url}\n\n{api_keys_str}", title="更新api-keys")


if __name__ == '__main__':
    cli()  # https://api.deepseek.com/user/balance https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf\?sheet\=X0ZN3H



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : flux
# @Time         : 2024/8/5 09:52
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import asyncio

from meutils.pipe import *
from meutils.config_utils.lark_utils import get_spreadsheet_values, get_next_token_for_polling
from meutils.schemas.openai_types import ImageRequest, ImagesResponse
from meutils.apis.translator import deeplx
from meutils.schemas.translator_types import DeeplxRequest
from meutils.decorators.retry import retrying
from meutils.schemas.image_types import ASPECT_RATIOS
from meutils.oss.minio_oss import Minio

from meutils.notice.feishu import send_message as _send_message

BASE_URL = "https://fluxpro.art"
FEISHU_URL = "https://xchatllm.feishu.cn/sheets/GYCHsvI4qhnDPNtI4VPcdw2knEd?sheet=GdRbM9"

send_message = partial(
    _send_message,
    title=__name__,
    url="https://open.feishu.cn/open-apis/bot/v2/hook/dc1eda96-348e-4cb5-9c7c-2d87d584ca18"
)


# import fake_useragent
#
# ua = fake_useragent.UserAgent()


@retrying(max_retries=5, title=__name__, predicate=lambda x: x is True)
async def create_image(request: ImageRequest, token: Optional[str] = None):
    token = token or await get_next_token_for_polling(feishu_url=FEISHU_URL)

    prompt = (await deeplx.translate(DeeplxRequest(text=request.prompt, target_lang="EN"))).get("data")

    payload = {
        "prompt": prompt,
        "negative_prompt": request.negative_prompt,
        "aspect_ratio": request.size if request.size in ASPECT_RATIOS else "1:1",

        "guidance": request.guidance_scale,
        "steps": request.num_inference_steps,
        "nsfw_level": request.nsfw_level  # 0 1 2 3
    }

    headers = {
        'Cookie': token,
        'User-Agent': "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/127.0.0.0 Safari/537.36"
    }
    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=100) as client:
        response = await client.post("/api/prompts/flux", json=payload)

        logger.debug(response.status_code)

        if response.status_code in {429, 403}:  # 触发重试
            logger.debug(f"{response.status_code} {token}")
            send_message(f"{response.status_code} {token}")
            return True

        if response.is_success:
            data = response.json().get('assets', [])
            image_data = []

            # asyncio.gather()

            async def atask(url):
                resp = await client.get(url, follow_redirects=True)
                file_object = await Minio().put_object_for_openai(
                    file=resp.content,
                    filename=f"{shortuuid.random()}.webp"
                )
                return {"url": file_object.filename, "revised_prompt": prompt}

            tasks = [atask(i.get('src')) for i in data]
            image_data = await asyncio.gather(*tasks)

            return ImagesResponse.construct(data=image_data)

        response.raise_for_status()


# {
#     "id": "clzianobv017nq200g3fd2zb1",
#     "prompt": "borttiful scenery nature glass bottle landscape, , purple galaxy seed",
#     "negative_prompt": "",
#     "aspect_ratio": "1:1",
#     "assets": [
#         {
#             "src": "/api/view/clzianobv017nq200g3fd2zb1/0.webp"
#         }
#     ],
#     "model": "FLUX.1 [pro]",
#     "created_at": "2024-08-06T10:45:19.723Z",
# }


if __name__ == '__main__':
    token="_ga=GA1.1.378655276.1724288763; next-auth.csrf-token=fc3c64918d60db1bc897b389c1b736dc6a4329f6675a485d428d7dd36d46695f%7C7d52961d266b22ef57ec398c4119f903c7a62c853eb69a9f4cc4742829bacce7; next-auth.callback-url=http%3A%2F%2Ffluxpro.art; next-auth.session-token=17394b70-92fc-43dc-ac7b-77a635076298; _ga_KJL6XMRTB6=GS1.1.1724293123.2.1.1724297068.0.0.0"
    arun(create_image(ImageRequest(prompt="画条狗", size='1024x512'), token=token))
    # arun(create_image(ImageRequest(prompt="画条狗"), token=token))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : chatbot
# @Time         : 2023/10/27 14:31
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://nicedouble-streamlitantdcomponentsdemo-app-middmy.streamlit.app/
from langchain.prompts import ChatPromptTemplate

from chatllm.llmchain.decorators import llm_stream
from meutils.pipe import *
from meutils.ai_cv.latex_ocr import latex_ocr
from langchain.chat_models import ChatOpenAI
from langchain.chains import LLMChain

import streamlit as st
from streamlit_extras.streaming_write import write




class ChatMessage(BaseModel):
    name: str = "assistant"  # "user", "assistant", or str
    avatar: Optional[str] = None
    generator: Any = '我是一条内容'


def chat_message(message: ChatMessage, help: Optional[str] = None, message_hook: Optional[Callable] = None):
    """
        chat_message(ChatMessage())
        chat_message(ChatMessage(name='assistant'))

        def chat_messages(messages: List[ChatMessage]):
            for msg in messages:
                chat_message(msg)

        chat_messages([ChatMessage()] * 10)
    """
    with st.chat_message(name=message.name, avatar=message.avatar):
        # message_placeholder = st.empty()
        # response = ''
        # for token in message.generator:
        #     # Display robot response in chat message container
        #     # time.sleep(0.1)
        #     # token = repr(f"""{token}""").strip("'")
        #     response += token
        #     message_placeholder.markdown(response + "▌")
        #
        # message_placeholder.markdown(response, unsafe_allow_html=True, help=help)

        # def fn():
        #     yield from message.generator

        write(message.generator)

        if message_hook: message_hook()


def ai_reply(user_input):
    template = ChatPromptTemplate.from_messages([
        ("system",
         """
         你是一名数学专家，你的名字叫火宝🔥
         要求：
         1. Let's think step by step. 
         2. 如果答案遇到公式请用标准的latex输出，公式用$包裹，例如：$\sqrt{{x^2+y^2}}=1$
         3. 务必用中文回答
         """.strip()),
        ('human', '开始解题：\n```{user_input}```')
    ])

    llm = LLMChain(llm=ChatOpenAI(model_name="gpt-4-0613", temperature=0, streaming=True), prompt=template)
    output = llm_stream(llm.run)(user_input=user_input)  # "gpt-3.5-"
    return output


if __name__ == '__main__':
    st.markdown('### 🔥解题小能手')

    # st.markdown(r':green[$\text{求极限}\lim_{x\to0}\frac{e^x-e^{-x}-2x}{x-\sin x}$]')
    # st.markdown(":green[$\sqrt{x^2+y^2}=1$] is a Pythagorean identity. :pencil:")
    # st.markdown(r'$\frac{d}{dx}(e^x + e^{-x} - 2) = e^x - e^{-x}] [\frac{d}{dx}(1 - \cos x) = \sin x$')

    # user_input = st.chat_input("    🤔 开始解题吧")

    file = st.file_uploader('上传题目图片', type=[".jpg", ".jpeg", '.png'])

    # 欢迎语
    chat_message(
        ChatMessage(generator='😘😘😘 嗨，我是你的解题小能手！\n\n **参考示例**：'),
        message_hook=lambda: st.latex(r"\lim_{x\to0}\frac{e^x-e^{-x}-2x}{x-\sin x}")
    )

    if file:
        with st.spinner('AI 正在思考🤔'):
            ocr_text = latex_ocr(file)
            # ocr_text_ = r"\lim_{x\to0}\frac{e^x-e^{-x}-2x}{x-\sin x}"
            chat_message(
                ChatMessage(generator='识别到的题目：\n\n'),
                message_hook=lambda: st.latex(ocr_text or "`未解析到具体问题`")
            )
        if ocr_text:
            with st.spinner('AI 正在解题🤔'):
                output = ai_reply(ocr_text)
                chat_message(ChatMessage(generator=output))

                # st.markdown(repr(f'{ocr_text}').strip("\'"))

            # st.markdown(_)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : hdfs_cmd
# @Time         : 2021/1/20 10:15 上午
# @Author       : yuanjie
# @Email        : meutils@qq.com
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *


class HDFS(object):
    HADOOP_HOME = os.environ.get('HADOOP_HOME', '~/infra-client/bin')
    HDFS_CLUSTER_NAME = os.environ.get('HDFS_CLUSTER_NAME', 'zjyprc-hadoop')

    HDFS_CMD = f"{HADOOP_HOME}/hdfs --cluster {HDFS_CLUSTER_NAME} dfs"  # f"{HADOOP_HOME}/hadoop --cluster {HDFS_CLUSTER_NAME} fs"

    @classmethod
    def check_path_isexist(cls, path):
        cmd = f"-test -e {path}"  # 包含？-test -d
        status, output = cls.magic_cmd(cmd)

        rst = False if status != 0 else True
        logger.info(f'Path Exist: {rst}')

        return rst

    @classmethod
    def touchz(cls, path):
        """

        :param path: /user/h_browser/algo/yuanjie/jars/xx.txt
        :return:
        """
        cmd = f"-touchz {path}"
        return cls.magic_cmd(cmd)

    @classmethod
    def wc_l(cls, path):
        """

        :param path: /user/h_data_platform/platform/browser/push_group/locals/江苏_南京/date=20210120/*
        :return:
        """
        cmd = f"-cat {path} | wc -l"
        return cls.magic_cmd(cmd)

    @classmethod
    def magic_cmd(cls, cmd):
        """

        :param cmd: -cat /user/h_browser/algo/yuanjie/jars/vec.csv
        :return:
        """
        cmd = f"{cls.HDFS_CMD} {cmd}"
        return magic_cmd(cmd)

    @classmethod
    def push2hdfs(cls, input, output):
        cls.magic_cmd(f"-mkdir -p {output}")
        cls.magic_cmd(f"-put -f {input} {output}")
        cls.touchz(f"{output}/_SUCCESS")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : edits
# @Time         : 2025/7/9 15:38
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :


from meutils.pipe import *
from meutils.apis.utils import make_request_httpx

FEISHU_URL = "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=gg5DNy"


async def create_task(image_url: str, api_key: Optional[str] = None):
    # 构造 multipart/form-data
    files = {
        "model": (None, "AnimeSharp"),
        "model_name": (None, "4x-UltraSharp"),
        "outscale": (None, "4"),
        "output_format": (None, "png"),
        "image_url": (None, image_url),
        "response_format": (None, "url")
    }
    headers = {
        "X-Failover-Enabled": "true",
        "Authorization": f"Bearer {api_key}",
        # "Content-Type": "application/x-www-form-urlencoded"
    }
    response = await make_request_httpx(
        base_url="https://ai.gitee.com/v1",
        path="/images/upscaling",
        files=files,  # todo 兼容性
        headers=headers,
        debug=True
    )

    return response


if __name__ == '__main__':
    image = "https://s3.ffire.cc/files/x.jpg"
    api_key = "AHKZ65ARNFH8QGMUUVCPECDTZVOLRPUXIKPGAC1Y"

    arun(create_task(image, api_key))
